import numpy as np
from PyQt5 import QtGui
from linien.client.widgets import CustomWidget


class CentralPanel(QtGui.QWidget, CustomWidget):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def connection_established(self):
        self.control = self.app().control
        params = self.app().parameters
        self.parameters = params

        params.ramp_amplitude.change(
            lambda value: self.ids.scan_amplitude.setText('%d %%' % (value * 100))
        )

        def change_auto_manual_mode(auto):
            self.get_widget('manual_navigation').setVisible(not auto)
            if auto:
                self.reset_scan_amplitude()

        params.automatic_mode.change(change_auto_manual_mode)

    def increase_scan_amplitude(self):
        self.change_range(True)

    def decrease_scan_amplitude(self):
        self.change_range(False)

    def go_right(self):
        self.change_center(True)

    def go_left(self):
        self.change_center(False)

    def change_center(self, positive):
        delta_center = self.parameters.ramp_amplitude.value / 10
        if not positive:
            delta_center *= -1
        new_center = self.parameters.center.value + delta_center

        if np.abs(new_center) + self.parameters.ramp_amplitude.value > 1:
            new_center = np.sign(new_center) * (1 - self.parameters.ramp_amplitude.value)

        self.parameters.center.value = new_center
        self.control.write_data()

    def change_range(self, positive):
        if positive:
            self.parameters.ramp_amplitude.value *= 1.5
        else:
            self.parameters.ramp_amplitude.value /= 1.5
        self.control.write_data()

    def reset_scan_amplitude(self):
        self.parameters.ramp_amplitude.reset()
        self.parameters.center.reset()
        self.control.write_data()


import numpy as np
from PyQt5 import QtGui
from linien.client.widgets import CustomWidget


class LockStatusPanel(QtGui.QWidget, CustomWidget):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def ready(self):
        self.ids.stop_lock.clicked.connect(self.stop_autolock)
        self.ids.control_signal_history_length.editingFinished.connect(self.control_signal_history_length_changed)

    def connection_established(self):
        self.control = self.app().control
        params = self.app().parameters
        self.parameters = params

        def update_status(_):
            locked = params.lock.value
            task = params.task.value
            al_failed = params.autolock_failed.value
            running = params.autolock_running.value

            if locked or (task is not None and not al_failed):
                self.show()
            else:
                self.hide()

            explain = not running

            if task:
                watching = params.autolock_watching.value
                failed = al_failed
            else:
                running = False
                watching = False
                failed = False

            def set_text(text):
                self.ids.lock_status.setText(text)

            if not running and locked:
                set_text('Locked!')
            if running and watching:
                set_text('Locked! Watching continuously...')
            if task and not running and failed:
                set_text('Autolock failed!')
            if running and not watching:
                set_text('Autolock is running...')

        for param in (params.lock, params.autolock_approaching, params.autolock_watching,
                params.autolock_failed, params.autolock_locked):
            param.change(update_status)

        params.control_signal_history_length.change(
            lambda value: self.ids.control_signal_history_length.setValue(value)
        )

    def stop_autolock(self):
        if self.parameters.task.value is not None:
            self.parameters.task.value.stop()
        else:
            self.control.exposed_start_ramp()

    def control_signal_history_length_changed(self):
        self.parameters.control_signal_history_length.value = \
            self.ids.control_signal_history_length.value()


import os
import paramiko

from linien.config import REMOTE_BASE_PATH


def run_server(host, user, password):
    ssh = paramiko.SSHClient()
    ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    ssh.connect(hostname=host, username=user, password=password)
    ftp = ssh.open_sftp()

    directory = os.path.abspath(
        os.path.join(
            os.path.dirname(__file__),
            '..'
        )
    )

    # upload the code required for running the server
    for dirpath, dirnames, filenames in os.walk(directory):
        dirpath_rel = dirpath.replace(directory, '').lstrip('/')
        remote_path = os.path.join(REMOTE_BASE_PATH, dirpath_rel)

        if '.' in dirpath_rel or '__' in dirpath_rel:
            continue

        try:
            ftp.lstat(remote_path)
        except IOError:
            ftp.mkdir(os.path.join(remote_path.rstrip('/')))

        for filename in filenames:
            local_path = os.path.join(dirpath, filename)
            remote_filepath = os.path.join(remote_path, filename)
            # put file
            ftp.put(local_path, remote_filepath)

    ftp.close()

    # start the server process
    ssh.exec_command('bash %s/server/start_server_in_screen.sh' % REMOTE_BASE_PATH)
    ssh.close()



from PyQt5 import QtGui
from linien.client.widgets import CustomWidget


class LockingPanel(QtGui.QWidget, CustomWidget):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def ready(self):
        self.ids.kp.editingFinished.connect(self.kp_changed)
        self.ids.ki.editingFinished.connect(self.ki_changed)
        self.ids.kd.editingFinished.connect(self.kd_changed)
        self.ids.watchLockCheckbox.stateChanged.connect(self.watch_lock_changed)
        self.ids.lock_control_container.currentChanged.connect(self.lock_mode_changed)

        self.ids.manualLockButton.clicked.connect(self.start_manual_lock)

    def connection_established(self):
        params = self.app().parameters
        self.parameters = params
        self.control = self.app().control

        params.p.change(
            lambda value: self.ids.kp.setValue(value)
        )
        params.i.change(
            lambda value: self.ids.ki.setValue(value)
        )
        params.d.change(
            lambda value: self.ids.kd.setValue(value)
        )
        params.watch_lock.change(
            lambda value: self.ids.watchLockCheckbox.setChecked(value)
        )
        params.automatic_mode.change(
            lambda value: self.ids.lock_control_container.setCurrentIndex(0 if value else 1)
        )

        def lock_status_changed(_):
            locked = params.lock.value
            task = params.task.value
            al_failed = params.autolock_failed.value
            task_running = (task is not None) and (not al_failed)

            if locked or task_running:
                self.ids.lock_control_container.hide()
            else:
                self.ids.lock_control_container.show()

        for param in (params.lock, params.autolock_approaching, params.autolock_watching,
                      params.autolock_failed, params.autolock_locked):
            param.change(lock_status_changed)

        def target_slope_changed(rising):
            self.ids.button_slope_rising.setChecked(rising)
            self.ids.button_slope_falling.setChecked(not rising)
        params.target_slope_rising.change(target_slope_changed)

    def kp_changed(self):
        self.parameters.p.value = self.ids.kp.value()

    def ki_changed(self):
        self.parameters.i.value = self.ids.ki.value()

    def kd_changed(self):
        self.parameters.d.value = self.ids.kd.value()

    def watch_lock_changed(self):
        self.parameters.watch_lock.value = int(self.ids.watchLockCheckbox.checkState())

    def lock_mode_changed(self, idx):
        self.parameters.automatic_mode.value = idx == 0

    def start_manual_lock(self):
        self.parameters.target_slope_rising.value = self.ids.button_slope_rising.isChecked()
        self.control.write_data()
        self.control.start_lock()


import os
import sys
sys.path += ['../../']
import _thread
import pickle

import rpyc
from rpyc.utils.server import ThreadedServer

from registers import Registers
from autolock import Autolock
from parameters import Parameters

from linien.config import SERVER_PORT
from linien.common import update_control_signal_history
from linien.communication.server import BaseService


class RedPitayaControlService(BaseService):
    def __init__(self):
        self._cached_data = {}
        self.exposed_is_locked = None

        super().__init__(Parameters)

        #self.registers = Registers(host='rp-f0685a.local', user='root', password='zeilinger')
        self.registers = Registers()
        self.registers.connect(self, self.parameters)

    def run_acquiry_loop(self):
        def on_change(plot_data):
            self.parameters.to_plot.value = plot_data
            self.parameters.control_signal_history.value = \
                update_control_signal_history(
                    self.parameters.control_signal_history.value,
                    pickle.loads(plot_data),
                    self.exposed_is_locked,
                    self.parameters.control_signal_history_length.value
                )

        self.registers.run_data_acquisition(on_change)

    def exposed_write_data(self):
        self.registers.write_registers()

    def exposed_start_autolock(self, x0, x1, spectrum):
        spectrum = pickle.loads(spectrum)
        start_watching = self.parameters.watch_lock.value

        if not self.parameters.autolock_running.value:
            autolock = Autolock(self, self.parameters)
            self.parameters.task.value = autolock
            autolock.run(x0, x1, spectrum, should_watch_lock=start_watching)

    def exposed_start_ramp(self):
        self.parameters.lock.value = False
        self.exposed_write_data()

    def exposed_start_lock(self):
        self.parameters.lock.value = True
        self.exposed_write_data()

    def exposed_reset(self):
        self.parameters.ramp_amplitude.value = 1
        self.parameters.center.value = 0
        self.exposed_start_ramp()
        self.exposed_write_data()

    def exposed_shutdown(self):
        self.registers.acquisition.shutdown()
        _thread.interrupt_main()
        os._exit(0)


class FakeRedPitayaControl(BaseService):
    def __init__(self):
        super().__init__(Parameters)
        self.exposed_is_locked = None

    def exposed_write_data(self):
        pass

    def run_acquiry_loop(self):
        import threading
        from time import sleep
        from random import randint

        def run():
            while True:
                self.parameters.to_plot.value = pickle.dumps((
                    [randint(-8192, 8192) for _ in range(16384)],
                    list(_ - 8192 for _ in range(16384))
                ))
                sleep(.1)
        t = threading.Thread(target=run)
        t.daemon = True
        t.start()

    def exposed_shutdown(self):
        _thread.interrupt_main()
        os._exit(0)

    def exposed_start_autolock(self, x0, x1, spectrum):
        print('start autolock', x0, x1)


if __name__ == '__main__':
    control = RedPitayaControlService()
    #control = FakeRedPitayaControl()
    control.run_acquiry_loop()
    control.exposed_write_data()

    t = ThreadedServer(control, port=SERVER_PORT)
    t.start()


import sys
sys.path += ['../../']
import rpyc
import atexit
import threading

from enum import Enum
from time import sleep
from multiprocessing import Process, Pipe

from linien.config import ACQUISITION_PORT
from linien.server.utils import start_acquisition_process, stop_nginx, \
    start_nginx, flash_fpga


class AcquisitionConnectionError(Exception):
    pass


class AcquisitionProcessSignals(Enum):
    SHUTDOWN = 0
    SET_ASG_OFFSET = 1
    SET_RAMP_SPEED = 2
    SET_LOCK_STATUS = 3


class AcquisitionMaster:
    def __init__(self, on_acquisition, use_ssh, host):
        def receive_acquired_data(conn):
            while True:
                on_acquisition(conn.recv())

        self.acq_process, child_pipe = Pipe()
        p = Process(
            target=self.connect_acquisition_process,
            args=(child_pipe, use_ssh, host)
        )
        p.start()

        # wait until connection is established
        self.acq_process.recv()

        t = threading.Thread(target=receive_acquired_data, args=(self.acq_process,))
        t.daemon = True
        t.start()

        atexit.register(self.shutdown)

    def connect_acquisition_process(self, pipe, use_ssh, host):
        if use_ssh:
            pitaya_rpyc = rpyc.connect(host, ACQUISITION_PORT)
        else:
            for i in range(2):
                try:
                    pitaya_rpyc = rpyc.connect('127.0.0.1', ACQUISITION_PORT)
                except:
                    if i == 0:
                        stop_nginx()
                        flash_fpga()
                        start_acquisition_process()

                        # FIXME: shorter?
                        sleep(2)
                    else:
                        raise AcquisitionConnectionError()

        # tell the main thread that we're ready
        pipe.send(True)

        # run a loop that listens for acquired data and transmits them
        # to the main thread. Also redirects calls from the main thread
        # to the acquiry process.
        while True:
            # check whether the main thread sent a command to the acquiry process
            if pipe.poll():
                data = pipe.recv()
                if data[0] == AcquisitionProcessSignals.SHUTDOWN:
                    break
                elif data[0] == AcquisitionProcessSignals.SET_ASG_OFFSET:
                    idx, value = data[1:]
                    pitaya_rpyc.root.set_asg_offset(idx, value)
                elif data[0] == AcquisitionProcessSignals.SET_RAMP_SPEED:
                    speed = data[1]
                    pitaya_rpyc.root.set_ramp_speed(speed)
                elif data[0] == AcquisitionProcessSignals.SET_LOCK_STATUS:
                    pitaya_rpyc.root.set_lock_status(data[1])

            # load acquired data and send it to the main thread
            data = pitaya_rpyc.root.return_data()
            pipe.send(data)

            sleep(0.05)

    def shutdown(self):
        if self.acq_process:
            self.acq_process.send((AcquisitionProcessSignals.SHUTDOWN,))

        start_nginx()

    def set_asg_offset(self, idx, offset):
        self.acq_process.send((AcquisitionProcessSignals.SET_ASG_OFFSET, idx, offset))

    def set_ramp_speed(self, speed):
        self.acq_process.send((AcquisitionProcessSignals.SET_RAMP_SPEED, speed))

    def lock_status_changed(self, status):
        if self.acq_process:
            self.acq_process.send((AcquisitionProcessSignals.SET_LOCK_STATUS, status))


import shutil
import subprocess

from linien.config import REMOTE_BASE_PATH


def stop_nginx():
    subprocess.Popen(['systemctl', 'stop', 'redpitaya_nginx.service']).wait()
    subprocess.Popen(['systemctl', 'stop', 'redpitaya_scpi.service']).wait()


def start_nginx():
    subprocess.Popen(['systemctl', 'start', 'redpitaya_nginx.service'])


def start_acquisition_process():
    subprocess.Popen(['python3', REMOTE_BASE_PATH + '/server/acquisition_process.py'])


def flash_fpga():
    shutil.copyfile('redpid.bin', '/dev/xdevcfg')


import numpy as np
from PyQt5 import QtGui
from linien.client.widgets import CustomWidget

MHz = 0x10000000 / 8
Vpp = ((1<<14) - 1) / 4

class SpectroscopyPanel(QtGui.QWidget, CustomWidget):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def ready(self):
        self.ids.modulation_frequency.editingFinished.connect(self.change_modulation_frequency)
        self.ids.modulation_amplitude.editingFinished.connect(self.change_modulation_amplitude)
        self.ids.signal_offset.editingFinished.connect(self.change_signal_offset)
        self.ids.ramp_speed.currentIndexChanged.connect(self.change_ramp_speed)
        self.ids.demodulation_phase.editingFinished.connect(self.change_demod_phase)
        self.ids.demodulation_frequency.currentIndexChanged.connect(self.change_demod_multiplier)

    def connection_established(self):
        params = self.app().parameters
        self.control = self.app().control
        self.parameters = params

        #self.close_button.clicked.connect(self.close_app)
        #self.shutdown_button.clicked.connect(self.shutdown_server)

        params.modulation_frequency.change(
            lambda value: self.ids.modulation_frequency.setValue(value / MHz)
        )

        params.modulation_amplitude.change(
            lambda value: self.ids.modulation_amplitude.setValue(value / Vpp)
        )

        params.ramp_speed.change(
            lambda value: self.ids.ramp_speed.setCurrentIndex(value)
        )

        params.demodulation_phase.change(
            lambda value: self.ids.demodulation_phase.setValue(value)
        )

        params.demodulation_multiplier.change(
            lambda value: self.ids.demodulation_frequency.setCurrentIndex(value - 1)
        )

        params.offset.change(
            lambda value: self.ids.signal_offset.setValue(value)
        )

    def change_modulation_frequency(self):
        self.parameters.modulation_frequency.value = self.ids.modulation_frequency.value() * MHz
        self.control.write_data()

    def change_modulation_amplitude(self):
        self.parameters.modulation_amplitude.value = self.ids.modulation_amplitude.value() * Vpp
        self.control.write_data()

    def change_signal_offset(self):
        self.parameters.offset.value = self.ids.signal_offset.value()
        self.control.write_data()

    def change_ramp_speed(self, decimation):
        self.parameters.ramp_speed.value = decimation
        self.control.write_data()

    def change_demod_phase(self):
        self.parameters.demodulation_phase.value = self.ids.demodulation_phase.value()
        self.control.write_data()

    def change_demod_multiplier(self, idx):
        self.parameters.demodulation_multiplier.value = idx + 1
        self.control.write_data()


import math
import pickle
import numpy as np
from PyQt5 import QtGui, QtWidgets
from linien.client.widgets import CustomWidget
from pyqtgraph.Qt import QtCore, QtGui
import pyqtgraph as pg
from time import time
from linien.common import update_control_signal_history, determine_shift_by_correlation, \
    get_lock_point, control_signal_has_correct_amplitude


class PlotWidget(pg.PlotWidget, CustomWidget):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.hideAxis('bottom')
        self.hideAxis('left')

        self.setMouseEnabled(x=False, y=False)
        self.setMenuEnabled(False)

        # important: increasing pen width makes plotting much slower!
        # alternative: pg.setConfigOptions(useOpenGL=True)
        # see: https://github.com/pyqtgraph/pyqtgraph/issues/533
        pen_width = 1

        self.zero_line = pg.PlotCurveItem(pen=pg.mkPen('w', width=1))
        self.addItem(self.zero_line)
        self.signal = pg.PlotCurveItem(pen=pg.mkPen((200, 0, 0, 200), width=pen_width))
        self.addItem(self.signal)
        self.control_signal = pg.PlotCurveItem(pen=pg.mkPen((0, 0, 200, 200), width=pen_width))
        self.addItem(self.control_signal)
        self.control_signal_history = pg.PlotCurveItem(pen=pg.mkPen((0, 200, 0, 200), width=pen_width))
        self.addItem(self.control_signal_history)

        self.zero_line.setData([0, 16383], [0, 0])
        self.signal.setData([0, 16383], [1, 1])

        self.connection = None
        self.parameters = None
        self.last_plot_rescale = 0
        self.last_plot_data = None
        self.plot_max = 0
        self.plot_min = np.inf
        self.touch_start = None
        self.autolock_ref_spectrum = None

        self.init_overlay()
        self.init_lock_target_line()

    def _to_data_coords(self, event):
        pos = self.plotItem.vb.mapSceneToView(event.pos())
        x, y = pos.x(), pos.y()
        return x, y

    def connection_established(self):
        self.control = self.app().control
        self.parameters = self.app().parameters

        self.control_signal_history_data = self.parameters.control_signal_history.value

        self.parameters.to_plot.change(self.replot)

    def mouseMoveEvent(self, event):
        if self.touch_start is None:
            return

        x0, y0 = self.touch_start

        x, y = self._to_data_coords(event)
        self.set_selection_overlay(x0, x - x0)

    def init_overlay(self):
        self.overlay = pg.LinearRegionItem(values=(-1000, -1000), movable=False)
        self.overlay.setVisible(False)
        self.addItem(self.overlay)

    def init_lock_target_line(self):
        self.lock_target_line = pg.InfiniteLine(movable=False)
        self.lock_target_line.setValue(1000)
        self.addItem(self.lock_target_line)

    def set_selection_overlay(self, x_start, width):
        self.overlay.setRegion((x_start, x_start + width))

    def mousePressEvent(self, event):
        super().mousePressEvent(event)
        x, y = self._to_data_coords(event)

        self.touch_start = x, y
        self.set_selection_overlay(x, 0)
        self.overlay.setVisible(True)

    def mouseReleaseEvent(self, event):
        super().mouseReleaseEvent(event)

        if self.touch_start is None:
            return

        x, y = self._to_data_coords(event)
        x0, y0 = self.touch_start
        xdiff = np.abs(x0 - x)

        if xdiff / self.xmax < 0.01:
            # it was a click
            if not self.parameters.automatic_mode.value:
                self.graph_on_click(x0, y0)
        else:
            # it was a selection
            if self.parameters.automatic_mode.value:
                last_error_signal = self.last_plot_data[0]
                self.control.start_autolock(
                    *sorted([x0, x]),
                    # we pickle it here because otherwise a netref is
                    # transmitted which blocks the autolock
                    pickle.dumps(last_error_signal)
                )
                mean_signal, target_slope_rising, target_zoom, rolled_error_signal = \
                    get_lock_point(last_error_signal, int(x0), int(x))
                self.autolock_ref_spectrum = rolled_error_signal
            else:
                self.graph_on_selection(x0, x)

        self.overlay.setVisible(False)
        self.touch_start = None

    @property
    def xmax(self):
        return len(self.last_plot_data[0]) - 1

    def graph_on_selection(self, x0, x):
        x0 /= self.xmax
        x /= self.xmax

        center = np.mean([x, x0])
        amplitude = np.abs(center - x) * 2
        center = (center - 0.5) * 2

        amplitude *= self.parameters.ramp_amplitude.value
        center = self.parameters.center.value + \
            (center * self.parameters.ramp_amplitude.value)

        self.parameters.ramp_amplitude.value = amplitude
        self.parameters.center.value = center
        self.control.write_data()

    def graph_on_click(self, x, y):
        center = x / self.xmax
        center = (center - 0.5) * 2
        center = self.parameters.center.value + \
            (center * self.parameters.ramp_amplitude.value)

        self.parameters.ramp_amplitude.value /= 2
        self.parameters.center.value = center
        self.control.write_data()

    def replot(self, to_plot):
        if to_plot is not None and not self.touch_start:
            to_plot = pickle.loads(to_plot)

            if to_plot is None:
                return

            self.last_plot_data = to_plot

            error_signal, control_signal = to_plot

            if not self.parameters.lock.value:
                ramp_amplitude = self.parameters.ramp_amplitude.value
                if not control_signal_has_correct_amplitude(control_signal, ramp_amplitude):
                    print('SKIPPING')
                    return

            self.plot_data(control_signal, error_signal)
            self.plot_autolock_target_line(error_signal)
            self.update_plot_scaling(error_signal)
            self.update_control_signal_history(to_plot)

    def plot_data(self, control_signal, error_signal):
        self.signal.setData(list(range(len(error_signal))), error_signal)
        self.control_signal.setData(
            list(range(len(error_signal))),
            [
                point / 8192 * self.plot_max
                for point in control_signal
            ]
        )

    def plot_autolock_target_line(self, error_signal):
        if self.autolock_ref_spectrum is not None and self.parameters.autolock_approaching.value:
            ramp_amplitude = self.parameters.ramp_amplitude.value
            zoom_factor = 1 / ramp_amplitude


            shift, _1, _2 = determine_shift_by_correlation(
                zoom_factor,
                self.autolock_ref_spectrum,
                error_signal
            )

            length = len(error_signal)
            shift = (length / 2) - (shift / 2* length * zoom_factor)

            self.lock_target_line.setVisible(True)
            self.lock_target_line.setValue(shift)
        else:
            self.lock_target_line.setVisible(False)


    def update_plot_scaling(self, error_signal):
        self.plot_max = np.max([-1 * self.plot_min, self.plot_max, math.ceil(np.max(error_signal))])
        self.plot_min = np.min([-1 * self.plot_max, self.plot_min, math.floor(np.min(error_signal))])

        if time() - self.last_plot_rescale > 2:
            plot_min = math.floor(self.plot_min)
            plot_max = math.ceil(self.plot_max)

            if plot_min == plot_max:
                plot_max += 1

            self.setYRange(plot_min, plot_max)

            self.last_plot_rescale = time()

    def update_control_signal_history(self, to_plot):
        self.control_history_data = update_control_signal_history(
            self.control_signal_history_data,
            to_plot,
            self.parameters.lock.value,
            self.parameters.control_signal_history_length.value
        )
        if self.parameters.lock.value:
            x_axis_length = 16384
            timescale = self.parameters.control_signal_history_length.value
            times = np.array(self.control_signal_history_data['times'])
            times -= times[0]
            times *= 1 / timescale * x_axis_length
            self.control_signal_history.setData(
                times,
                [
                    point / 8192 * self.plot_max
                    for point in self.control_signal_history_data['values']
                ]
            )
            self.control_signal_history.setVisible(True)
        else:
            self.control_signal_history.setVisible(False)


import os
import sys
import numpy as np

from PyQt5 import QtWidgets
from pyqtgraph.Qt import QtCore, QtGui
# add ui folder to path
sys.path += [
    os.path.join(*list(
        os.path.split(os.path.abspath(__file__))[:-1]) + ['ui']
    )
]

from linien.client.widgets import CustomWidget
from linien.client.ui.main_window import Ui_MainWindow
from linien.client.ui.device_manager import Ui_DeviceManager


class QTApp(QtCore.QObject):
    ready = QtCore.pyqtSignal(bool)

    def __init__(self):
        self.app = QtWidgets.QApplication(sys.argv)

        self.main_window = QtWidgets.QMainWindow()
        self.main_window.app = self
        ui = Ui_MainWindow()
        ui.setupUi(self.main_window)

        self.device_manager = QtWidgets.QMainWindow()
        self.device_manager.app = self
        ui2 = Ui_DeviceManager()
        ui2.setupUi(self.device_manager)
        self.device_manager.show()

        self.app.aboutToQuit.connect(lambda: self.app.quit())

        super().__init__()

    def connected(self, connection, parameters, control):
        self.device_manager.hide()
        self.main_window.show()

        self.connection = connection
        self.control = control
        self.parameters = parameters

        self.ready.connect(self.init)
        self.ready.emit(True)

    def init(self):
        for instance in CustomWidget.instances:
            instance.connection_established()

        self.parameters.call_listeners()

    def get_widget(self, name, window=None):
        """Queries a widget by name."""
        window = window or self.main_window
        return window.findChild(QtCore.QObject, name)

    def close(self):
        self.app.quit()

    def shutdown(self):
        self.control.shutdown()
        self.close()

    def open_device_manager(self):
        self.main_window.hide()
        self.device_manager.show()

        self.connection.disconnect()
        del self.connection


from linien.communication.server import Parameter, BaseParameters


class Parameters(BaseParameters):
    def __init__(self):
        super().__init__()

        self.modulation_amplitude = Parameter(
            min_=0,
            max_=(1<<14) - 1,
            start=4046
        )
        self.modulation_frequency = Parameter(
            min_=0,
            max_=0xffffffff,
            # 0x10000000 ~= 8 MHzs
            start=0x10000000/8*15
        )
        self.center = Parameter(
            min_=-1,
            max_=1,
            start=0
        )
        self.offset = Parameter(
            min_=-8191,
            max_=8191,
            start=0
        )
        self.ramp_amplitude = Parameter(
            min_=0.001,
            max_=1,
            start=1
        )
        self.ramp_speed = Parameter(
            min_=0,
            max_=16,
            start=9
        )
        self.demodulation_phase = Parameter(
            min_=0,
            max_=360,
            start=0x0,
            wrap=True
        )
        self.demodulation_multiplier = Parameter(
            min_=0,
            max_=15,
            start=1
        )
        self.lock = Parameter(start=False)
        self.to_plot = Parameter()

        self.p = Parameter(start=50)
        self.i = Parameter(start=5)
        self.d = Parameter(start=0)
        self.task = Parameter(start=None, sync=False)
        self.automatic_mode = Parameter(start=True)
        self.target_slope_rising = Parameter(start=True)
        self.autolock_running = Parameter(start=False)
        self.autolock_approaching = Parameter(start=False)
        self.autolock_watching = Parameter(start=False)
        self.autolock_failed = Parameter(start=False)
        self.autolock_locked = Parameter(start=False)

        self.watch_lock = Parameter(start=True)
        self.control_signal_history = Parameter(start={
            'times': [],
            'values': []
        }, sync=False)
        # in seconds
        self.control_signal_history_length = Parameter(start=600)


import os
import sys
import pickle
import _thread
import threading
from time import sleep, time
from rpyc import Service
from rpyc.utils.server import OneShotServer
from PyRedPitaya.board import RedPitaya

sys.path += ['../../']
from linien.config import ACQUISITION_PORT


def shutdown():
    _thread.interrupt_main()
    os._exit(0)


class DataAcquisitionService(Service):
    def __init__(self):
        self.r = RedPitaya()

        self.data = pickle.dumps(None)
        self.data_retrieval_time = None

        super(DataAcquisitionService, self).__init__()

        self.exposed_set_ramp_speed(9)
        self.locked = False
        self.start_time = time()

        self.run()

    def run(self, trigger_delay=16384):
        def run_acquiry_loop():
            while True:
                # copied from https://github.com/RedPitaya/RedPitaya/blob/14cca62dd58f29826ee89f4b28901602f5cdb1d8/api/src/oscilloscope.c#L115
                # check whether scope was triggered
                if (self.r.scope.read(0x1<<2) & 0x4) > 0 and not self.locked:
                    sleep(.05)
                    continue

                data = [
                    [float(i) for i in channel[:]]
                    for channel in
                    (self.r.scope.data_ch1, self.r.scope.data_ch2)
                ]
                self.r.scope.rearm(trigger_source=6)
                self.r.scope.data_decimation = 2 ** self.ramp_speed
                self.r.scope.trigger_delay = trigger_delay - 1
                self.data = pickle.dumps(data)

                if self.data_retrieval_time is None and (time() - self.start_time > 5):
                    # acquisition process is up and running but server did not poll
                    # anything. Maybe it died, so should we
                    print('neverpoll')
                    return shutdown()

                if self.data_retrieval_time is not None:
                    # FIXME: increased this timeout, still necessary?
                    if time() - self.data_retrieval_time > 5:
                        print('nopoll')
                        # the parent process did not poll for more than 2 seconds.
                        # This probably means that it died, so shut down this
                        # child process, too
                        return shutdown()

        self.t = threading.Thread(target=run_acquiry_loop, args=())
        self.t.daemon = True
        self.t.start()

    def exposed_return_data(self):
        self.data_retrieval_time = time()
        return self.data[:]

    def exposed_set_asg_offset(self, idx, value):
        asg = getattr(self.r, ['asga', 'asgb'][idx])
        asg.offset = value

    def exposed_set_ramp_speed(self, speed):
        self.ramp_speed = speed

    def exposed_set_lock_status(self, locked):
        self.locked = locked

if __name__ == '__main__':
    t = OneShotServer(DataAcquisitionService(), port=ACQUISITION_PORT)
    t.start()


from PyQt5 import QtGui
from linien.client.config import load_device_data, save_device_data
from linien.client.widgets import CustomWidget


class NewDeviceDialogCenter(QtGui.QWidget, CustomWidget):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def add_new_device(self):
        device = {
            'name': self.ids.deviceName.text(),
            'host': self.ids.host.text(),
            'username': self.ids.username.text(),
            'password': self.ids.password.text()
        }
        devices = load_device_data() + [device]
        save_device_data(devices)


import numpy as np
from time import sleep, time

from csr import make_filter, PitayaLocal, PitayaSSH
from utils import start_nginx, stop_nginx
from linien.config import DEFAULT_RAMP_SPEED
from linien.server.acquisition import AcquisitionMaster


class Registers:
    def __init__(self, host=None, user=None, password=None):
        self.host = host
        self.user = user
        self.password = password
        self.acquisition = None

    def connect(self, control, parameters):
        self.control = control
        self.parameters = parameters

        self.use_ssh = self.host is not None and self.host not in ('localhost', '127.0.0.1')

        if self.use_ssh:
            self.rp = PitayaSSH(
                ssh_cmd="sshpass -p %s ssh %s@%s" % (self.password, self.user, self.host)
            )
        else:
            self.rp = PitayaLocal()

        def lock_status_changed(v):
            if self.acquisition is not None:
                self.acquisition.lock_status_changed(v)

        self.parameters.lock.change(lock_status_changed)

    def write_registers(self):
        params = dict(self.parameters)

        _max = lambda val: val if np.abs(val) <= 8191 else (8191 * val / np.abs(val))
        sweep_min = -1 * _max(params['ramp_amplitude'] * 8191)
        sweep_max = _max(params['ramp_amplitude'] * 8191)

        demod_delay = int(
            params['demodulation_phase'] / 360 * (1<<14)
        )
        demod_multiplier = params['demodulation_multiplier']

        new = dict(
            # channel B (channel for ramping and PID)
            fast_b_x_tap=2,
            fast_b_demod_delay=demod_delay,
            fast_b_demod_multiplier=demod_multiplier,
            fast_b_brk=0,
            fast_b_dx_sel=self.rp.signal("scopegen_dac_a"),
            fast_b_y_tap=4,

            fast_b_sweep_run=1,
            fast_b_sweep_step=int(
                DEFAULT_RAMP_SPEED * params['ramp_amplitude']
                / (2 ** params['ramp_speed'])
            ),
            fast_b_sweep_min=sweep_min,
            fast_b_sweep_max=sweep_max,
            fast_b_dy_sel=self.rp.signal("scopegen_dac_b"),

            fast_b_mod_freq=params['modulation_frequency'],
            fast_b_mod_amp=0x0,

            fast_b_relock_run=0,
            fast_b_relock_en=self.rp.states(),
            fast_b_y_hold_en=self.rp.states(),
            fast_b_y_clear_en=self.rp.states(),
            fast_b_rx_sel=self.rp.signal('zero'),

            # channel A (channel for modulation)
            fast_a_brk=1,
            fast_a_mod_amp=params['modulation_amplitude'],
            fast_a_mod_freq=params['modulation_frequency'],
            fast_a_x_tap=2,
            fast_a_demod_delay=demod_delay,
            fast_a_demod_multiplier=demod_multiplier,
            fast_a_sweep_run=0,
            fast_a_pid_kp=0,
            fast_a_pid_ki=0,
            fast_a_pid_kd=0,
            fast_a_dy_sel=self.rp.signal('zero'),

            scopegen_adc_a_sel=self.rp.signal("fast_b_x"),
            scopegen_adc_b_sel=self.rp.signal("fast_b_y"),
            # trigger on ramp
            scopegen_external_trigger=2,

            gpio_p_oes=0,
            gpio_n_oes=0,

            gpio_p_outs=0,
            gpio_n_outs=0,

            gpio_n_do0_en=self.rp.signal('zero'),
            gpio_n_do1_en=self.rp.signal('zero'),

            # asg offset (is not set via ssh but via rpyc)
            asga_offset=int(params['offset']),
            asgb_offset=int(params['center'] * 8191),
        )

        lock_changed = params['lock'] != self.control.exposed_is_locked
        lock = params['lock']
        self.control.exposed_is_locked = lock

        new['fast_b_sweep_run'] = 0 if lock else 1

        # filter out values that did not change
        new = dict(
            (k, v)
            for k, v in new.items()
            if (
                (k not in self.control._cached_data)
                or (self.control._cached_data.get(k) != v)
            )
        )
        self.control._cached_data.update(new)

        # set ASG offset
        for idx, asg in enumerate(('asga', 'asgb')):
            try:
                value = new.pop('%s_offset' % asg)
                self.acquisition.set_asg_offset(idx, value)
            except KeyError:
                pass

        # pass ramp speed changes to acquisition process
        if 'fast_b_sweep_step' in new:
            self.acquisition.set_ramp_speed(params['ramp_speed'])

        for k, v in new.items():
            self.rp.set(k, int(v))

        if 'fast_b_sweep_step' in new:
            # reset sweep for a short time if the scan range was changed
            # this is needed because otherwise it may take too long before
            # the new scan range is reached --> no scope trigger is sent
            self.rp.set('fast_b_sweep_run', 0)
            self.rp.set('fast_b_sweep_run', 1)

        kp = params['p']
        ki = params['i']
        kd = params['d']
        slope = params['target_slope_rising']

        if lock_changed:
            if lock:
                # sync modulation phases
                self.sync_modulation_phases()

                # set PI parameters
                self.set_pid(kp, ki, kd, slope, reset=0)
            else:
                self.set_pid(0, 0, 0, slope, reset=1)

                self.rp.set_iir("fast_a_iir_a", *make_filter('P', k=1))
                self.rp.set_iir("fast_a_iir_c", *make_filter("P", k=0))
                self.rp.set_iir("fast_b_iir_a", *make_filter('P', k=1))
                self.rp.set_iir("fast_b_iir_c", *make_filter("P", k=0))

        else:
            # hold PID value
            self.hold_pid(True)

            if lock:
                # set new PI parameters
                self.set_pid(kp, ki, kd, slope)

            # reset "hold"
            self.hold_pid(False)

        self.sync_modulation_phases()

    def sync_modulation_phases(self):
        self.rp.set('root_sync_phase_en', self.rp.states('force'))
        self.rp.set('root_sync_phase_en', self.rp.states())

    def run_data_acquisition(self, on_change):
        self.acquisition = AcquisitionMaster(
            on_change, self.use_ssh, self.host
        )

    def set_pid(self, p, i, d, slope, reset=None):
        sign = -1 if slope else 1
        self.rp.set('fast_b_pid_kp', p * sign)
        self.rp.set('fast_b_pid_ki', i * sign)
        self.rp.set('fast_b_pid_kd', d * sign)

        if reset is not None:
            self.rp.set('fast_b_pid_reset', reset)

    def hold_pid(self, hold):
        self.rp.set(
            'fast_b_y_hold_en',
            self.rp.states('force') if hold else self.rp.states()
        )



import os
import pickle
import appdirs


def get_data_folder():
    folder_name = appdirs.user_data_dir('linien')

    if not os.path.exists(folder_name):
        os.mkdir(folder_name)

    return folder_name


def get_devices_filename():
    return os.path.join(
        get_data_folder(),
        'devices'
    )


def save_device_data(devices):
    with open(get_devices_filename(), 'wb') as f:
        pickle.dump(devices, f)


def load_device_data():
    try:
        with open(get_devices_filename(), 'rb') as f:
            devices = pickle.load(f)
    except FileNotFoundError:
        devices = []

    return devices


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'new_device_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_NewDeviceDialog(object):
    def setupUi(self, NewDeviceDialog):
        NewDeviceDialog.setObjectName("NewDeviceDialog")
        NewDeviceDialog.resize(318, 292)
        self.horizontalLayout = QtWidgets.QHBoxLayout(NewDeviceDialog)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.newDeviceDialogCenter = NewDeviceDialogCenter(NewDeviceDialog)
        self.newDeviceDialogCenter.setObjectName("newDeviceDialogCenter")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.newDeviceDialogCenter)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.groupBox_4 = QtWidgets.QGroupBox(self.newDeviceDialogCenter)
        self.groupBox_4.setObjectName("groupBox_4")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.groupBox_4)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.deviceName = QtWidgets.QLineEdit(self.groupBox_4)
        self.deviceName.setObjectName("deviceName")
        self.verticalLayout_5.addWidget(self.deviceName)
        self.verticalLayout.addWidget(self.groupBox_4)
        self.groupBox = QtWidgets.QGroupBox(self.newDeviceDialogCenter)
        self.groupBox.setObjectName("groupBox")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.groupBox)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.host = QtWidgets.QLineEdit(self.groupBox)
        self.host.setObjectName("host")
        self.horizontalLayout_2.addWidget(self.host)
        self.verticalLayout.addWidget(self.groupBox)
        self.groupBox_2 = QtWidgets.QGroupBox(self.newDeviceDialogCenter)
        self.groupBox_2.setObjectName("groupBox_2")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox_2)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.username = QtWidgets.QLineEdit(self.groupBox_2)
        self.username.setObjectName("username")
        self.verticalLayout_3.addWidget(self.username)
        self.verticalLayout.addWidget(self.groupBox_2)
        self.groupBox_3 = QtWidgets.QGroupBox(self.newDeviceDialogCenter)
        self.groupBox_3.setObjectName("groupBox_3")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.groupBox_3)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.password = QtWidgets.QLineEdit(self.groupBox_3)
        self.password.setObjectName("password")
        self.verticalLayout_4.addWidget(self.password)
        self.verticalLayout.addWidget(self.groupBox_3)
        self.horizontalLayout_3.addLayout(self.verticalLayout)
        self.horizontalLayout.addWidget(self.newDeviceDialogCenter)
        self.buttonBox = QtWidgets.QDialogButtonBox(NewDeviceDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Vertical)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.horizontalLayout.addWidget(self.buttonBox)

        self.retranslateUi(NewDeviceDialog)
        self.buttonBox.accepted.connect(NewDeviceDialog.accept)
        self.buttonBox.rejected.connect(NewDeviceDialog.reject)
        self.buttonBox.accepted.connect(self.newDeviceDialogCenter.add_new_device)
        QtCore.QMetaObject.connectSlotsByName(NewDeviceDialog)

    def retranslateUi(self, NewDeviceDialog):
        _translate = QtCore.QCoreApplication.translate
        NewDeviceDialog.setWindowTitle(_translate("NewDeviceDialog", "Dialog"))
        self.groupBox_4.setTitle(_translate("NewDeviceDialog", "Name"))
        self.groupBox.setTitle(_translate("NewDeviceDialog", "Host"))
        self.host.setText(_translate("NewDeviceDialog", "rp-xxxxxx.local"))
        self.groupBox_2.setTitle(_translate("NewDeviceDialog", "Username"))
        self.username.setText(_translate("NewDeviceDialog", "root"))
        self.groupBox_3.setTitle(_translate("NewDeviceDialog", "Password"))
        self.password.setText(_translate("NewDeviceDialog", "root"))


from new_device_dialog_center import NewDeviceDialogCenter


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    NewDeviceDialog = QtWidgets.QDialog()
    ui = Ui_NewDeviceDialog()
    ui.setupUi(NewDeviceDialog)
    NewDeviceDialog.show()
    sys.exit(app.exec_())



from time import sleep
from socket import gaierror

from linien.config import SERVER_PORT
from linien.client.utils import run_server
from linien.communication.client import BaseClient


class ConnectionError(Exception):
    pass


class Connection(BaseClient):
    def __init__(self, host, user=None, password=None):
        self.user = user
        self.password = password

        if host in ('localhost', '127.0.0.1'):
            # RP is configured such that "localhost" doesn't point to
            # 127.0.0.1 in all cases
            host = '127.0.0.1'
        else:
            assert user and password

        super().__init__(host, SERVER_PORT)

        self.control = self.connection.root

    def connect(self, host, port):
        self.connection = None

        i = -1

        while True:
            i += 1

            try:
                print('try to connect', host, port)
                self._connect(host, port)
                break
            except gaierror:
                # host not found
                print('host not found')
                break
            except Exception as e:
                print('connection error', e)

                if i == 0:
                    print('start server')
                    try:
                        run_server(host, self.user, self.password)
                        sleep(7)
                    except:
                        print('starting server failed')
                        sleep(1)
                        continue

            sleep(1)

            if i == 10:
                break

        if self.connection is None:
            raise ConnectionError()

        print('connected', host, port)

    def disconnect(self):
        self.connection.close()


from PyQt5 import QtGui, QtWidgets, QtCore
from linien.client.config import load_device_data, save_device_data
from linien.client.widgets import CustomWidget
from linien.client.ui.new_device_dialog import Ui_NewDeviceDialog
from linien.client.connection import Connection


class DeviceManagerCenter(QtGui.QWidget, CustomWidget):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def ready(self):
        self.load_device_data(autoload=True)

    def load_device_data(self, autoload=False):
        devices = load_device_data()
        lst = self.ids.deviceList
        lst.clear()

        for device in devices:
            lst.addItem('%s (%s)' % (device['name'], device['host']))

        if autoload and len(devices) == 1:
            self.connect_to_device(devices[0])

    def connect(self):
        devices = load_device_data()

        if not devices:
            return

        self.connect_to_device(devices[self.get_list_index()])

    def connect_to_device(self, device):
        try:
            conn = Connection(device['host'], device['username'], device['password'])
            self.app().connected(conn, conn.parameters, conn.control)
        except:
            pass

    def new_device(self):
        self.dialog = QtWidgets.QDialog()
        ui = Ui_NewDeviceDialog()
        ui.setupUi(self.dialog)
        self.dialog.setModal(True)
        self.dialog.show()

        def reload_device_data():
            # not very elegant...
            QtCore.QTimer.singleShot(100, self.load_device_data)

        self.dialog.accepted.connect(reload_device_data)

    def get_list_index(self):
        return self.ids.deviceList.currentIndex().row()

    def remove_device(self):
        devices = load_device_data()

        if not devices:
            return

        devices.pop(self.get_list_index())
        save_device_data(devices)
        self.load_device_data()

    def selected_device_changed(self):
        idx = self.get_list_index()

        disable_buttons = True

        if idx >= 0:
            devices = load_device_data()

            if devices:
                disable_buttons = False

        for btn in [self.ids.connectButton, self.ids.removeButton]:
            btn.setEnabled(not disable_buttons)


import pickle
import traceback
import numpy as np
from time import sleep, time
from linien.common import determine_shift_by_correlation, get_lock_point, \
    control_signal_has_correct_amplitude


class Autolock:
    """Spectroscopy autolock based on correlation."""
    def __init__(self, control, parameters):
        self.control = control
        self.parameters = parameters

        self.first_error_signal = None
        self.parameters.autolock_running.value = False
        self.should_watch_lock = False

        self.reset_properties()

    def reset_properties(self):
        self.history = []
        self.zoom_factor = 1
        self.N_at_this_zoom = 0
        self.skipped = 0
        self.parameters.autolock_failed.value = False
        self.parameters.autolock_locked.value = False
        self.parameters.autolock_watching.value = False

    def run(self, x0, x1, spectrum, should_watch_lock=False):
        """Starts the autolock.

        If `should_watch_lock` is specified, the autolock continuously monitors
        the control and error signals after the lock was successful and tries to
        relock automatically using the spectrum that was recorded in the first
        run of the lock.
        """
        self.parameters.autolock_running.value = True
        self.x0, self.x1 = int(x0), int(x1)
        self.should_watch_lock = should_watch_lock

        self.parameters.autolock_approaching.value = True
        self.record_first_error_signal(spectrum)

        self.add_data_listener()

    def add_data_listener(self):
        self.parameters.to_plot.change(self.react_to_new_spectrum)

    def remove_data_listener(self):
        self.parameters.to_plot.remove_listener(self.react_to_new_spectrum)

    def react_to_new_spectrum(self, plot_data):
        """React to new spectrum data.

        If this is executed for the first time, a reference spectrum is
        recorded.

        If the autolock is approaching the desired line, a correlation
        function of the spectrum with the reference spectrum is calculated
        and the laser current is adapted such that the targeted line is centered.

        After this procedure is done, the real lock is turned on and after some
        time the lock is verified.

        If automatic relocking is desired, the control and error signals are
        continuously monitored after locking.
        """
        if plot_data is None or not self.parameters.autolock_running.value:
            return

        plot_data = pickle.loads(plot_data)
        if plot_data is None:
            return

        error_signal, control_signal = plot_data

        try:
            if self.parameters.autolock_approaching.value:
                # we have already recorded a spectrum and are now approaching
                # the line by decreasing the scan range and adapting the
                # center current multiple times.

                if self.skipped < 10:
                    # after every step, we skip some data in order to let
                    # the laser equilibrate
                    self.skipped += 1
                    return

                self.skipped = 0

                return self.approach_line(error_signal, control_signal)

            elif self.parameters.autolock_watching.value:
                # the laser was locked successfully before. Now we check
                # periodically whether the laser is still in lock

                return self.watch_lock(error_signal, control_signal)

            else:
                # we are done with approaching and have started the lock.
                # skip some data and check whether we really are in lock
                # afterwards.

                if self.skipped < 10:
                    self.skipped += 1
                    return

                self.skipped = 0

                return self.after_lock(control_signal)

        except Exception:
            traceback.print_exc()
            self.exposed_stop()

    def record_first_error_signal(self, error_signal):
        # FIXME: Should this only be allowed when fully zoomed out?
        mean_signal, target_slope_rising, target_zoom, rolled_error_signal = \
            get_lock_point(error_signal, self.x0, self.x1)

        self.parameters.offset.value -= mean_signal
        self.parameters.target_slope_rising.value = target_slope_rising
        self.control.exposed_write_data()

        self.target_zoom = target_zoom
        self.first_error_signal = rolled_error_signal

    def approach_line(self, error_signal, control_signal):
        # check that the data received is new data, i.e. with the correct
        # scan range. Otherwise just wait for the next sample.
        if control_signal_has_correct_amplitude(control_signal, self.parameters.ramp_amplitude.value):
            shift, zoomed_ref, zoomed_err = determine_shift_by_correlation(
                self.zoom_factor, self.first_error_signal, error_signal
            )
            self.history.append((zoomed_ref, zoomed_err))
            print('N', self.N_at_this_zoom, 'SHIFT', shift)

            self.control.exposed_write_data()
            self.history.append('shift %f' % (-1 * shift))

            self.parameters.center.value -= shift
            self.control.exposed_write_data()

            self.N_at_this_zoom += 1

            if self.N_at_this_zoom > 1:
                self.N_at_this_zoom = 0

                zoom_step = 2
                self.zoom_factor *= zoom_step
                self.parameters.ramp_amplitude.value /= zoom_step
                self.control.exposed_write_data()

                if self.zoom_factor >= self.target_zoom:
                    print('START LOCK!')
                    self.parameters.autolock_approaching.value = False
                    self.control.exposed_start_lock()

    def after_lock(self, control_signal):
        """After locking, this method checks whether the laser really is locked.

        If desired, it automatically tries to relock if locking failed, or
        starts a watcher that does so over and over again.
        """
        def check_whether_in_lock(control_signal):
            """
            The laser is considered in lock if the mean value of the control
            signal is within the boundaries of the smalles current ramp we had
            before turning on the lock.
            """
            mean = np.mean(control_signal) / 8192
            center = self.parameters.center.value
            ampl = self.parameters.ramp_amplitude.value
            return (center - ampl) <= mean <= (center + ampl)

        self.parameters.autolock_locked.value = check_whether_in_lock(control_signal)

        if self.parameters.autolock_locked.value and self.should_watch_lock:
            # we start watching the lock status from now on.
            # this is done in `react_to_new_spectrum()` which is called regularly.
            self.parameters.autolock_watching.value = True
        else:
            self.remove_data_listener()

            if not self.parameters.autolock_locked.value:
                if self.should_watch_lock:
                    return self.relock()

                self.control.exposed_reset()
                self.parameters.autolock_failed.value = True
                self.remove_data_listener()

            self.parameters.autolock_running.value = False

    def watch_lock(self, error_signal, control_signal):
        """Check whether the laser is still in lock and init a relock if not."""
        mean = np.abs(np.mean(control_signal) / 8192)
        still_in_lock = mean < 0.9

        if not still_in_lock:
            self.relock()

    def relock(self):
        """
        Relock the laser using the reference spectrum recorded in the first
        locking approach.
        """
        self.reset_properties()
        self.parameters.autolock_running.value = True
        self.parameters.autolock_approaching.value = True

        self.parameters.center.value = 0
        self.parameters.ramp_amplitude.value = 1
        self.control.exposed_start_ramp()

        # add a listener that listens for new spectrum data and consequently
        # tries to relock.
        self.add_data_listener()

    def exposed_stop(self):
        """Abort any operation."""
        self.parameters.autolock_failed.value = True
        self.parameters.autolock_running.value = False
        self.parameters.autolock_locked.value = False
        self.parameters.autolock_approaching.value = False
        self.parameters.autolock_watching.value = False
        self.remove_data_listener()

        self.control.exposed_reset()


"""
    communication.client
    ~~~~~~~~~~~~~~~~~~~~

    Contains the client that can be used to access a service with easy access
    to the server's parameters.
"""
import rpyc
import uuid
from .remote_parameters import RemoteParameters


class ClientService(rpyc.Service):
    def __init__(self, uuid):
        super().__init__()

        self.exposed_uuid = uuid


class BaseClient:
    def __init__(self, server, port):
        self.uuid = uuid.uuid4().hex

        self.client_service = ClientService(self.uuid)

        self.connect(server, port)

    def connect(self, server, port):
        return self._connect(server, port)

    def _connect(self, server, port):
        self.connection = rpyc.connect(server, port, service=self.client_service)
        self.parameters = RemoteParameters(
            self.connection.root,
            self.uuid
        )


csr_constants = {
    'fast_a_iir_a_shift': 16,
    'fast_a_iir_a_width': 18,
    'fast_a_iir_a_interval': 1,
    'fast_a_iir_a_latency': 2,
    'fast_a_iir_a_order': 1,
    'fast_a_iir_a_iterative': 0,
    'fast_a_iir_b_shift': 23,
    'fast_a_iir_b_width': 25,
    'fast_a_iir_b_interval': 5,
    'fast_a_iir_b_latency': 6,
    'fast_a_iir_b_order': 2,
    'fast_a_iir_b_iterative': 1,
    'fast_a_iir_c_shift': 16,
    'fast_a_iir_c_width': 18,
    'fast_a_iir_c_interval': 1,
    'fast_a_iir_c_latency': 2,
    'fast_a_iir_c_order': 1,
    'fast_a_iir_c_iterative': 0,
    'fast_a_iir_d_shift': 16,
    'fast_a_iir_d_width': 18,
    'fast_a_iir_d_interval': 1,
    'fast_a_iir_d_latency': 3,
    'fast_a_iir_d_order': 2,
    'fast_a_iir_d_iterative': 0,
    'fast_a_iir_e_shift': 23,
    'fast_a_iir_e_width': 25,
    'fast_a_iir_e_interval': 5,
    'fast_a_iir_e_latency': 6,
    'fast_a_iir_e_order': 2,
    'fast_a_iir_e_iterative': 1,
    'fast_a_relock_shift': 16,
    'fast_a_sweep_shift': 18,
    'fast_b_iir_a_shift': 16,
    'fast_b_iir_a_width': 18,
    'fast_b_iir_a_interval': 1,
    'fast_b_iir_a_latency': 2,
    'fast_b_iir_a_order': 1,
    'fast_b_iir_a_iterative': 0,
    'fast_b_iir_b_shift': 23,
    'fast_b_iir_b_width': 25,
    'fast_b_iir_b_interval': 5,
    'fast_b_iir_b_latency': 6,
    'fast_b_iir_b_order': 2,
    'fast_b_iir_b_iterative': 1,
    'fast_b_iir_c_shift': 16,
    'fast_b_iir_c_width': 18,
    'fast_b_iir_c_interval': 1,
    'fast_b_iir_c_latency': 2,
    'fast_b_iir_c_order': 1,
    'fast_b_iir_c_iterative': 0,
    'fast_b_iir_d_shift': 16,
    'fast_b_iir_d_width': 18,
    'fast_b_iir_d_interval': 1,
    'fast_b_iir_d_latency': 3,
    'fast_b_iir_d_order': 2,
    'fast_b_iir_d_iterative': 0,
    'fast_b_iir_e_shift': 23,
    'fast_b_iir_e_width': 25,
    'fast_b_iir_e_interval': 5,
    'fast_b_iir_e_latency': 6,
    'fast_b_iir_e_order': 2,
    'fast_b_iir_e_iterative': 1,
    'fast_b_relock_shift': 16,
    'fast_b_sweep_shift': 18,
    'slow_a_iir_shift': 23,
    'slow_a_iir_width': 25,
    'slow_a_iir_interval': 75,
    'slow_a_iir_latency': 6,
    'slow_a_iir_order': 2,
    'slow_a_iir_iterative': 1,
    'slow_b_iir_shift': 23,
    'slow_b_iir_width': 25,
    'slow_b_iir_interval': 75,
    'slow_b_iir_latency': 6,
    'slow_b_iir_order': 2,
    'slow_b_iir_iterative': 1,
    'slow_c_iir_shift': 23,
    'slow_c_iir_width': 25,
    'slow_c_iir_interval': 75,
    'slow_c_iir_latency': 6,
    'slow_c_iir_order': 2,
    'slow_c_iir_iterative': 1,
    'slow_d_iir_shift': 23,
    'slow_d_iir_width': 25,
    'slow_d_iir_interval': 75,
    'slow_d_iir_latency': 6,
    'slow_d_iir_order': 2,
    'slow_d_iir_iterative': 1,
}

csr = {
    'dna_dna': (28, 0x000, 64, False),
    'fast_a_x_tap': (0, 0x000, 2, True),
    'fast_a_brk': (0, 0x001, 1, True),
    'fast_a_y_tap': (0, 0x002, 3, True),
    'fast_a_pid_setpoint': (0, 0x003, 14, True),
    'fast_a_pid_kp': (0, 0x005, 14, True),
    'fast_a_pid_ki': (0, 0x007, 14, True),
    'fast_a_pid_reset': (0, 0x009, 1, True),
    'fast_a_pid_kd': (0, 0x00a, 14, True),
    'fast_a_iir_a_z0': (0, 0x00c, 27, True),
    'fast_a_iir_a_a1': (0, 0x010, 18, True),
    'fast_a_iir_a_b0': (0, 0x013, 18, True),
    'fast_a_iir_a_b1': (0, 0x016, 18, True),
    'fast_a_demod_delay': (0, 0x019, 32, True),
    'fast_a_demod_multiplier': (0, 0x01d, 4, True),
    'fast_a_iir_b_z0': (0, 0x01e, 38, True),
    'fast_a_iir_b_a1': (0, 0x023, 25, True),
    'fast_a_iir_b_a2': (0, 0x027, 25, True),
    'fast_a_iir_b_b0': (0, 0x02b, 25, True),
    'fast_a_iir_b_b1': (0, 0x02f, 25, True),
    'fast_a_iir_b_b2': (0, 0x033, 25, True),
    'fast_a_x_limit_min': (0, 0x037, 25, True),
    'fast_a_x_limit_max': (0, 0x03b, 25, True),
    'fast_a_iir_c_z0': (0, 0x03f, 27, True),
    'fast_a_iir_c_a1': (0, 0x043, 18, True),
    'fast_a_iir_c_b0': (0, 0x046, 18, True),
    'fast_a_iir_c_b1': (0, 0x049, 18, True),
    'fast_a_iir_d_z0': (0, 0x04c, 27, True),
    'fast_a_iir_d_a1': (0, 0x050, 18, True),
    'fast_a_iir_d_a2': (0, 0x053, 18, True),
    'fast_a_iir_d_b0': (0, 0x056, 18, True),
    'fast_a_iir_d_b1': (0, 0x059, 18, True),
    'fast_a_iir_d_b2': (0, 0x05c, 18, True),
    'fast_a_iir_e_z0': (0, 0x05f, 38, True),
    'fast_a_iir_e_a1': (0, 0x064, 25, True),
    'fast_a_iir_e_a2': (0, 0x068, 25, True),
    'fast_a_iir_e_b0': (0, 0x06c, 25, True),
    'fast_a_iir_e_b1': (0, 0x070, 25, True),
    'fast_a_iir_e_b2': (0, 0x074, 25, True),
    'fast_a_relock_step': (0, 0x078, 24, True),
    'fast_a_relock_run': (0, 0x07b, 1, True),
    'fast_a_relock_min': (0, 0x07c, 15, True),
    'fast_a_relock_max': (0, 0x07e, 15, True),
    'fast_a_sweep_step': (0, 0x080, 24, True),
    'fast_a_sweep_min': (0, 0x083, 14, True),
    'fast_a_sweep_max': (0, 0x085, 14, True),
    'fast_a_sweep_run': (0, 0x087, 1, True),
    'fast_a_mod_amp': (0, 0x088, 14, True),
    'fast_a_mod_freq': (0, 0x08a, 32, True),
    'fast_a_y_limit_min': (0, 0x08e, 14, True),
    'fast_a_y_limit_max': (0, 0x090, 14, True),
    'fast_a_x_clr': (0, 0x092, 1, True),
    'fast_a_x_max': (0, 0x093, 25, False),
    'fast_a_x_min': (0, 0x097, 25, False),
    'fast_a_y_clr': (0, 0x09b, 1, True),
    'fast_a_y_max': (0, 0x09c, 25, False),
    'fast_a_y_min': (0, 0x0a0, 25, False),
    'fast_a_sweep_trigger_clr': (0, 0x0a4, 1, True),
    'fast_a_sweep_trigger_max': (0, 0x0a5, 1, False),
    'fast_a_sweep_trigger_min': (0, 0x0a6, 1, False),
    'fast_a_x_hold_en': (0, 0x0a7, 27, True),
    'fast_a_x_clear_en': (0, 0x0ab, 27, True),
    'fast_a_y_hold_en': (0, 0x0af, 27, True),
    'fast_a_y_clear_en': (0, 0x0b3, 27, True),
    'fast_a_relock_en': (0, 0x0b7, 27, True),
    'fast_a_dx_sel': (0, 0x0bb, 5, True),
    'fast_a_dy_sel': (0, 0x0bc, 5, True),
    'fast_a_rx_sel': (0, 0x0bd, 5, True),
    'fast_b_x_tap': (1, 0x000, 2, True),
    'fast_b_brk': (1, 0x001, 1, True),
    'fast_b_y_tap': (1, 0x002, 3, True),
    'fast_b_pid_setpoint': (1, 0x003, 14, True),
    'fast_b_pid_kp': (1, 0x005, 14, True),
    'fast_b_pid_ki': (1, 0x007, 14, True),
    'fast_b_pid_reset': (1, 0x009, 1, True),
    'fast_b_pid_kd': (1, 0x00a, 14, True),
    'fast_b_iir_a_z0': (1, 0x00c, 27, True),
    'fast_b_iir_a_a1': (1, 0x010, 18, True),
    'fast_b_iir_a_b0': (1, 0x013, 18, True),
    'fast_b_iir_a_b1': (1, 0x016, 18, True),
    'fast_b_demod_delay': (1, 0x019, 32, True),
    'fast_b_demod_multiplier': (1, 0x01d, 4, True),
    'fast_b_iir_b_z0': (1, 0x01e, 38, True),
    'fast_b_iir_b_a1': (1, 0x023, 25, True),
    'fast_b_iir_b_a2': (1, 0x027, 25, True),
    'fast_b_iir_b_b0': (1, 0x02b, 25, True),
    'fast_b_iir_b_b1': (1, 0x02f, 25, True),
    'fast_b_iir_b_b2': (1, 0x033, 25, True),
    'fast_b_x_limit_min': (1, 0x037, 25, True),
    'fast_b_x_limit_max': (1, 0x03b, 25, True),
    'fast_b_iir_c_z0': (1, 0x03f, 27, True),
    'fast_b_iir_c_a1': (1, 0x043, 18, True),
    'fast_b_iir_c_b0': (1, 0x046, 18, True),
    'fast_b_iir_c_b1': (1, 0x049, 18, True),
    'fast_b_iir_d_z0': (1, 0x04c, 27, True),
    'fast_b_iir_d_a1': (1, 0x050, 18, True),
    'fast_b_iir_d_a2': (1, 0x053, 18, True),
    'fast_b_iir_d_b0': (1, 0x056, 18, True),
    'fast_b_iir_d_b1': (1, 0x059, 18, True),
    'fast_b_iir_d_b2': (1, 0x05c, 18, True),
    'fast_b_iir_e_z0': (1, 0x05f, 38, True),
    'fast_b_iir_e_a1': (1, 0x064, 25, True),
    'fast_b_iir_e_a2': (1, 0x068, 25, True),
    'fast_b_iir_e_b0': (1, 0x06c, 25, True),
    'fast_b_iir_e_b1': (1, 0x070, 25, True),
    'fast_b_iir_e_b2': (1, 0x074, 25, True),
    'fast_b_relock_step': (1, 0x078, 24, True),
    'fast_b_relock_run': (1, 0x07b, 1, True),
    'fast_b_relock_min': (1, 0x07c, 15, True),
    'fast_b_relock_max': (1, 0x07e, 15, True),
    'fast_b_sweep_step': (1, 0x080, 24, True),
    'fast_b_sweep_min': (1, 0x083, 14, True),
    'fast_b_sweep_max': (1, 0x085, 14, True),
    'fast_b_sweep_run': (1, 0x087, 1, True),
    'fast_b_mod_amp': (1, 0x088, 14, True),
    'fast_b_mod_freq': (1, 0x08a, 32, True),
    'fast_b_y_limit_min': (1, 0x08e, 14, True),
    'fast_b_y_limit_max': (1, 0x090, 14, True),
    'fast_b_x_clr': (1, 0x092, 1, True),
    'fast_b_x_max': (1, 0x093, 25, False),
    'fast_b_x_min': (1, 0x097, 25, False),
    'fast_b_y_clr': (1, 0x09b, 1, True),
    'fast_b_y_max': (1, 0x09c, 25, False),
    'fast_b_y_min': (1, 0x0a0, 25, False),
    'fast_b_sweep_trigger_clr': (1, 0x0a4, 1, True),
    'fast_b_sweep_trigger_max': (1, 0x0a5, 1, False),
    'fast_b_sweep_trigger_min': (1, 0x0a6, 1, False),
    'fast_b_x_hold_en': (1, 0x0a7, 27, True),
    'fast_b_x_clear_en': (1, 0x0ab, 27, True),
    'fast_b_y_hold_en': (1, 0x0af, 27, True),
    'fast_b_y_clear_en': (1, 0x0b3, 27, True),
    'fast_b_relock_en': (1, 0x0b7, 27, True),
    'fast_b_dx_sel': (1, 0x0bb, 5, True),
    'fast_b_dy_sel': (1, 0x0bc, 5, True),
    'fast_b_rx_sel': (1, 0x0bd, 5, True),
    'gpio_n_ins': (30, 0x000, 8, False),
    'gpio_n_outs': (30, 0x001, 8, True),
    'gpio_n_oes': (30, 0x002, 8, True),
    'gpio_n_state': (30, 0x003, 27, False),
    'gpio_n_state_clr': (30, 0x007, 1, True),
    'gpio_n_do0_en': (30, 0x008, 27, True),
    'gpio_n_do1_en': (30, 0x00c, 27, True),
    'gpio_n_do2_en': (30, 0x010, 27, True),
    'gpio_n_do3_en': (30, 0x014, 27, True),
    'gpio_n_do4_en': (30, 0x018, 27, True),
    'gpio_n_do5_en': (30, 0x01c, 27, True),
    'gpio_n_do6_en': (30, 0x020, 27, True),
    'gpio_n_do7_en': (30, 0x024, 27, True),
    'gpio_p_ins': (31, 0x000, 8, False),
    'gpio_p_outs': (31, 0x001, 8, True),
    'gpio_p_oes': (31, 0x002, 8, True),
    'noise_bits': (7, 0x000, 5, True),
    'noise_y_clr': (7, 0x001, 1, True),
    'noise_y_max': (7, 0x002, 25, False),
    'noise_y_min': (7, 0x006, 25, False),
    'root_sync_phase_en': (8, 0x000, 27, True),
    'scopegen_external_trigger': (6, 0x000, 2, True),
    'scopegen_dac_a_clr': (6, 0x001, 1, True),
    'scopegen_dac_a_max': (6, 0x002, 25, False),
    'scopegen_dac_a_min': (6, 0x006, 25, False),
    'scopegen_dac_b_clr': (6, 0x00a, 1, True),
    'scopegen_dac_b_max': (6, 0x00b, 25, False),
    'scopegen_dac_b_min': (6, 0x00f, 25, False),
    'scopegen_adc_a_sel': (6, 0x013, 5, True),
    'scopegen_adc_b_sel': (6, 0x014, 5, True),
    'slow_a_brk': (2, 0x000, 1, True),
    'slow_a_x_limit_min': (2, 0x001, 25, True),
    'slow_a_x_limit_max': (2, 0x005, 25, True),
    'slow_a_iir_z0': (2, 0x009, 38, True),
    'slow_a_iir_a1': (2, 0x00e, 25, True),
    'slow_a_iir_a2': (2, 0x012, 25, True),
    'slow_a_iir_b0': (2, 0x016, 25, True),
    'slow_a_iir_b1': (2, 0x01a, 25, True),
    'slow_a_iir_b2': (2, 0x01e, 25, True),
    'slow_a_y_limit_min': (2, 0x022, 16, True),
    'slow_a_y_limit_max': (2, 0x024, 16, True),
    'slow_a_x_clr': (2, 0x026, 1, True),
    'slow_a_x_max': (2, 0x027, 25, False),
    'slow_a_x_min': (2, 0x02b, 25, False),
    'slow_a_y_clr': (2, 0x02f, 1, True),
    'slow_a_y_max': (2, 0x030, 25, False),
    'slow_a_y_min': (2, 0x034, 25, False),
    'slow_a_hold_en': (2, 0x038, 27, True),
    'slow_a_clear_en': (2, 0x03c, 27, True),
    'slow_a_dx_sel': (2, 0x040, 5, True),
    'slow_b_brk': (3, 0x000, 1, True),
    'slow_b_x_limit_min': (3, 0x001, 25, True),
    'slow_b_x_limit_max': (3, 0x005, 25, True),
    'slow_b_iir_z0': (3, 0x009, 38, True),
    'slow_b_iir_a1': (3, 0x00e, 25, True),
    'slow_b_iir_a2': (3, 0x012, 25, True),
    'slow_b_iir_b0': (3, 0x016, 25, True),
    'slow_b_iir_b1': (3, 0x01a, 25, True),
    'slow_b_iir_b2': (3, 0x01e, 25, True),
    'slow_b_y_limit_min': (3, 0x022, 16, True),
    'slow_b_y_limit_max': (3, 0x024, 16, True),
    'slow_b_x_clr': (3, 0x026, 1, True),
    'slow_b_x_max': (3, 0x027, 25, False),
    'slow_b_x_min': (3, 0x02b, 25, False),
    'slow_b_y_clr': (3, 0x02f, 1, True),
    'slow_b_y_max': (3, 0x030, 25, False),
    'slow_b_y_min': (3, 0x034, 25, False),
    'slow_b_hold_en': (3, 0x038, 27, True),
    'slow_b_clear_en': (3, 0x03c, 27, True),
    'slow_b_dx_sel': (3, 0x040, 5, True),
    'slow_c_brk': (4, 0x000, 1, True),
    'slow_c_x_limit_min': (4, 0x001, 25, True),
    'slow_c_x_limit_max': (4, 0x005, 25, True),
    'slow_c_iir_z0': (4, 0x009, 38, True),
    'slow_c_iir_a1': (4, 0x00e, 25, True),
    'slow_c_iir_a2': (4, 0x012, 25, True),
    'slow_c_iir_b0': (4, 0x016, 25, True),
    'slow_c_iir_b1': (4, 0x01a, 25, True),
    'slow_c_iir_b2': (4, 0x01e, 25, True),
    'slow_c_y_limit_min': (4, 0x022, 16, True),
    'slow_c_y_limit_max': (4, 0x024, 16, True),
    'slow_c_x_clr': (4, 0x026, 1, True),
    'slow_c_x_max': (4, 0x027, 25, False),
    'slow_c_x_min': (4, 0x02b, 25, False),
    'slow_c_y_clr': (4, 0x02f, 1, True),
    'slow_c_y_max': (4, 0x030, 25, False),
    'slow_c_y_min': (4, 0x034, 25, False),
    'slow_c_hold_en': (4, 0x038, 27, True),
    'slow_c_clear_en': (4, 0x03c, 27, True),
    'slow_c_dx_sel': (4, 0x040, 5, True),
    'slow_d_brk': (5, 0x000, 1, True),
    'slow_d_x_limit_min': (5, 0x001, 25, True),
    'slow_d_x_limit_max': (5, 0x005, 25, True),
    'slow_d_iir_z0': (5, 0x009, 38, True),
    'slow_d_iir_a1': (5, 0x00e, 25, True),
    'slow_d_iir_a2': (5, 0x012, 25, True),
    'slow_d_iir_b0': (5, 0x016, 25, True),
    'slow_d_iir_b1': (5, 0x01a, 25, True),
    'slow_d_iir_b2': (5, 0x01e, 25, True),
    'slow_d_y_limit_min': (5, 0x022, 16, True),
    'slow_d_y_limit_max': (5, 0x024, 16, True),
    'slow_d_x_clr': (5, 0x026, 1, True),
    'slow_d_x_max': (5, 0x027, 25, False),
    'slow_d_x_min': (5, 0x02b, 25, False),
    'slow_d_y_clr': (5, 0x02f, 1, True),
    'slow_d_y_max': (5, 0x030, 25, False),
    'slow_d_y_min': (5, 0x034, 25, False),
    'slow_d_hold_en': (5, 0x038, 27, True),
    'slow_d_clear_en': (5, 0x03c, 27, True),
    'slow_d_dx_sel': (5, 0x040, 5, True),
    'xadc_temp': (29, 0x000, 12, False),
    'xadc_v': (29, 0x002, 12, False),
    'xadc_a': (29, 0x004, 12, False),
    'xadc_b': (29, 0x006, 12, False),
    'xadc_c': (29, 0x008, 12, False),
    'xadc_d': (29, 0x00a, 12, False),
}
states = ['force', 'di0', 'di1', 'di2', 'di3', 'di4', 'di5', 'di6', 'di7', 'fast_a_x_sat', 'fast_a_x_railed', 'fast_a_y_sat', 'fast_a_y_railed', 'fast_a_unlocked', 'fast_b_x_sat', 'fast_b_x_railed', 'fast_b_y_sat', 'fast_b_y_railed', 'fast_b_unlocked', 'slow_a_sat', 'slow_a_railed', 'slow_b_sat', 'slow_b_railed', 'slow_c_sat', 'slow_c_railed', 'slow_d_sat', 'slow_d_railed']
signals = ['zero', 'fast_a_x', 'fast_a_y', 'fast_a_sweep_trigger', 'fast_b_x', 'fast_b_y', 'fast_b_sweep_trigger', 'slow_a_x', 'slow_a_y', 'slow_b_x', 'slow_b_y', 'slow_c_x', 'slow_c_y', 'slow_d_x', 'slow_d_y', 'scopegen_dac_a', 'scopegen_dac_b', 'noise_y']



from PyQt5 import QtGui
from linien.client.widgets import CustomWidget


class RightPanel(QtGui.QWidget, CustomWidget):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def connection_established(self):
        self.control = self.app().control
        self.parameters = self.app().parameters

    def close_app(self):
        self.app().close()

    def shutdown_server(self):
        self.app().shutdown()

    def open_device_manager(self):
        self.app().open_device_manager()


import os
import sys
import threading

sys.path += ['../', '../../']

from linien.client.gui import QTApp


def run_application():
    gui = QTApp()

    sys.exit(gui.app.exec_())


if __name__ == '__main__':
    run_application()



import numpy as np
from time import time
from scipy.signal import correlate, resample


def update_control_signal_history(history, to_plot, is_locked, max_time_diff):
    if not to_plot:
        return history

    error_signal, control_signal = to_plot
    now = time()

    if is_locked:
        history['values'].append(np.mean(control_signal))
        history['times'].append(time())
    else:
        history['values'] = []
        history['times'] = []

    # truncate
    while len(history['values']) > 0:
        if time() - history['times'][0] > max_time_diff:
            history['times'].pop(0)
            history['values'].pop(0)
            continue
        break

    return history


def determine_shift_by_correlation(zoom_factor, reference_signal, error_signal):
    length = len(error_signal)
    center_idx = int(length / 2)

    idx_shift = int(length * (1 / zoom_factor / 2))
    zoomed_ref = reference_signal[center_idx - idx_shift:center_idx + idx_shift]

    # correlation is slow on red pitaya --> use at maximum 4096 points
    skip_factor = int(len(zoomed_ref) / 4096)
    if skip_factor < 1:
        skip_factor = 1

    zoomed_ref = zoomed_ref[::skip_factor]
    downsampled_error_signal = resample(error_signal, len(zoomed_ref))

    correlation = correlate(zoomed_ref, downsampled_error_signal)
    """plt.plot(downsampled_error_signal)
    plt.plot(zoomed_ref)
    plt.show()"""


    """plt.plot(correlation)
    plt.show()"""

    shift = np.argmax(correlation)
    shift = (shift - len(zoomed_ref)) / len(zoomed_ref) * 2 / zoom_factor

    return shift, zoomed_ref, downsampled_error_signal


def get_lock_point(error_signal, x0, x1):
    cropped_data = np.array(error_signal[x0:x1])
    min_idx = np.argmin(cropped_data)
    max_idx = np.argmax(cropped_data)

    mean_signal = np.mean([cropped_data[min_idx], cropped_data[max_idx]])
    slope_data = np.array(cropped_data[min_idx:max_idx]) - mean_signal

    zero_idx = x0 + min_idx + np.argmin(np.abs(slope_data))
    target_slope_rising = max_idx > min_idx
    target_zoom = 16384 / (max_idx - min_idx) / 1.5

    length = len(error_signal)
    rolled_error_signal = np.roll(error_signal, -int(zero_idx - (length/2)))

    return mean_signal, target_slope_rising, target_zoom, rolled_error_signal


def control_signal_has_correct_amplitude(control_signal, amplitude_target):
    # we ignore some points as sometimes the triggering is not 100%ly
    # correct, i.e. at the beginning or end of the sample we have a glitch
    edge = int(0.4 * len(control_signal))
    control_signal_center = control_signal[edge:-edge]
    control_signal_amplitude = (
        np.max(control_signal_center) - np.min(control_signal_center)
    ) / len(control_signal_center) * len(control_signal) / 16384

    return np.abs(control_signal_amplitude - amplitude_target) / control_signal_amplitude < 0.1


import setuptools

with open("README.rst", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="linien",
    version="0.0.1",
    author="Benjamin Wiegand",
    author_email="highwaychile@posteo.de",
    description="Spectroscopy lock application using RedPitaya",
    long_description=long_description,
    long_description_content_type="text/x-rst",
    url="https://github.com/hermitdemschoenenleben/linien",
    packages=['linien', 'linien.communication', 'linien.client', 'linien.client.ui', 'linien.server'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    entry_points={
        'gui_scripts': [
            'linien=linien.client.client:run_application',
        ],
    },
    include_package_data=True
)


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'main_window.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1024, 768)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.centralwidget)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.centralPanel = CentralPanel(self.centralwidget)
        self.centralPanel.setObjectName("centralPanel")
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout(self.centralPanel)
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.verticalLayout_17 = QtWidgets.QVBoxLayout()
        self.verticalLayout_17.setObjectName("verticalLayout_17")
        self.manual_navigation = QtWidgets.QWidget(self.centralPanel)
        self.manual_navigation.setObjectName("manual_navigation")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.manual_navigation)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.scan_amplitude = QtWidgets.QLabel(self.manual_navigation)
        self.scan_amplitude.setObjectName("scan_amplitude")
        self.horizontalLayout_3.addWidget(self.scan_amplitude)
        self.decrease_scan_amplitude = QtWidgets.QPushButton(self.manual_navigation)
        self.decrease_scan_amplitude.setObjectName("decrease_scan_amplitude")
        self.horizontalLayout_3.addWidget(self.decrease_scan_amplitude)
        self.increase_scan_amplitude = QtWidgets.QPushButton(self.manual_navigation)
        self.increase_scan_amplitude.setObjectName("increase_scan_amplitude")
        self.horizontalLayout_3.addWidget(self.increase_scan_amplitude)
        self.reset_scan_amplitude = QtWidgets.QPushButton(self.manual_navigation)
        self.reset_scan_amplitude.setObjectName("reset_scan_amplitude")
        self.horizontalLayout_3.addWidget(self.reset_scan_amplitude)
        spacerItem = QtWidgets.QSpacerItem(195, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem)
        self.go_left = QtWidgets.QPushButton(self.manual_navigation)
        self.go_left.setObjectName("go_left")
        self.horizontalLayout_3.addWidget(self.go_left)
        self.go_right = QtWidgets.QPushButton(self.manual_navigation)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.go_right.sizePolicy().hasHeightForWidth())
        self.go_right.setSizePolicy(sizePolicy)
        self.go_right.setObjectName("go_right")
        self.horizontalLayout_3.addWidget(self.go_right)
        self.verticalLayout_17.addWidget(self.manual_navigation)
        self.graphicsView = PlotWidget(self.centralPanel)
        self.graphicsView.setMinimumSize(QtCore.QSize(100, 100))
        self.graphicsView.setObjectName("graphicsView")
        self.verticalLayout_17.addWidget(self.graphicsView)
        self.horizontalLayout_8.addLayout(self.verticalLayout_17)
        self.horizontalLayout.addWidget(self.centralPanel)
        self.rightPanel = RightPanel(self.centralwidget)
        self.rightPanel.setMinimumSize(QtCore.QSize(250, 0))
        self.rightPanel.setObjectName("rightPanel")
        self.verticalLayout_11 = QtWidgets.QVBoxLayout(self.rightPanel)
        self.verticalLayout_11.setObjectName("verticalLayout_11")
        self.openDeviceManagerButton = QtWidgets.QPushButton(self.rightPanel)
        self.openDeviceManagerButton.setObjectName("openDeviceManagerButton")
        self.verticalLayout_11.addWidget(self.openDeviceManagerButton)
        self.closeButton = QtWidgets.QPushButton(self.rightPanel)
        self.closeButton.setObjectName("closeButton")
        self.verticalLayout_11.addWidget(self.closeButton)
        self.shutdownButton = QtWidgets.QPushButton(self.rightPanel)
        palette = QtGui.QPalette()
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.WindowText, brush)
        brush = QtGui.QBrush(QtGui.QColor(204, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Button, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 51, 51))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Light, brush)
        brush = QtGui.QBrush(QtGui.QColor(229, 25, 25))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Midlight, brush)
        brush = QtGui.QBrush(QtGui.QColor(102, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Dark, brush)
        brush = QtGui.QBrush(QtGui.QColor(136, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Mid, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Text, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.BrightText, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.ButtonText, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Base, brush)
        brush = QtGui.QBrush(QtGui.QColor(204, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Window, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Shadow, brush)
        brush = QtGui.QBrush(QtGui.QColor(229, 127, 127))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.AlternateBase, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 220))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.ToolTipBase, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.ToolTipText, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 0, 128))
        brush.setStyle(QtCore.Qt.NoBrush)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.PlaceholderText, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.WindowText, brush)
        brush = QtGui.QBrush(QtGui.QColor(204, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Button, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 51, 51))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Light, brush)
        brush = QtGui.QBrush(QtGui.QColor(229, 25, 25))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Midlight, brush)
        brush = QtGui.QBrush(QtGui.QColor(102, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Dark, brush)
        brush = QtGui.QBrush(QtGui.QColor(136, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Mid, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Text, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.BrightText, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.ButtonText, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Base, brush)
        brush = QtGui.QBrush(QtGui.QColor(204, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Window, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Shadow, brush)
        brush = QtGui.QBrush(QtGui.QColor(229, 127, 127))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.AlternateBase, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 220))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.ToolTipBase, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.ToolTipText, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 0, 128))
        brush.setStyle(QtCore.Qt.NoBrush)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.PlaceholderText, brush)
        brush = QtGui.QBrush(QtGui.QColor(102, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.WindowText, brush)
        brush = QtGui.QBrush(QtGui.QColor(204, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Button, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 51, 51))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Light, brush)
        brush = QtGui.QBrush(QtGui.QColor(229, 25, 25))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Midlight, brush)
        brush = QtGui.QBrush(QtGui.QColor(102, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Dark, brush)
        brush = QtGui.QBrush(QtGui.QColor(136, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Mid, brush)
        brush = QtGui.QBrush(QtGui.QColor(102, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Text, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.BrightText, brush)
        brush = QtGui.QBrush(QtGui.QColor(102, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.ButtonText, brush)
        brush = QtGui.QBrush(QtGui.QColor(204, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Base, brush)
        brush = QtGui.QBrush(QtGui.QColor(204, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Window, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Shadow, brush)
        brush = QtGui.QBrush(QtGui.QColor(204, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.AlternateBase, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 220))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.ToolTipBase, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.ToolTipText, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 0, 128))
        brush.setStyle(QtCore.Qt.NoBrush)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.PlaceholderText, brush)
        self.shutdownButton.setPalette(palette)
        self.shutdownButton.setAutoFillBackground(False)
        self.shutdownButton.setObjectName("shutdownButton")
        self.verticalLayout_11.addWidget(self.shutdownButton)
        self.toolBox = QtWidgets.QToolBox(self.rightPanel)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.toolBox.sizePolicy().hasHeightForWidth())
        self.toolBox.setSizePolicy(sizePolicy)
        self.toolBox.setMinimumSize(QtCore.QSize(250, 0))
        self.toolBox.setObjectName("toolBox")
        self.page = SpectroscopyPanel()
        self.page.setGeometry(QtCore.QRect(0, 0, 227, 82))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.page.sizePolicy().hasHeightForWidth())
        self.page.setSizePolicy(sizePolicy)
        self.page.setObjectName("page")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.page)
        self.verticalLayout.setObjectName("verticalLayout")
        self.scrollArea_2 = QtWidgets.QScrollArea(self.page)
        self.scrollArea_2.setWidgetResizable(True)
        self.scrollArea_2.setObjectName("scrollArea_2")
        self.scrollAreaWidgetContents_3 = QtWidgets.QWidget()
        self.scrollAreaWidgetContents_3.setGeometry(QtCore.QRect(0, 0, 211, 500))
        self.scrollAreaWidgetContents_3.setObjectName("scrollAreaWidgetContents_3")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout(self.scrollAreaWidgetContents_3)
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.groupBox_4 = QtWidgets.QGroupBox(self.scrollAreaWidgetContents_3)
        self.groupBox_4.setObjectName("groupBox_4")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.groupBox_4)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.modulation_frequency = QtWidgets.QDoubleSpinBox(self.groupBox_4)
        self.modulation_frequency.setObjectName("modulation_frequency")
        self.horizontalLayout_6.addWidget(self.modulation_frequency)
        self.label_5 = QtWidgets.QLabel(self.groupBox_4)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_6.addWidget(self.label_5)
        self.verticalLayout_5.addLayout(self.horizontalLayout_6)
        self.verticalLayout_9.addWidget(self.groupBox_4)
        self.groupBox = QtWidgets.QGroupBox(self.scrollAreaWidgetContents_3)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.modulation_amplitude = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.modulation_amplitude.setMaximum(2.0)
        self.modulation_amplitude.setSingleStep(0.1)
        self.modulation_amplitude.setObjectName("modulation_amplitude")
        self.horizontalLayout_2.addWidget(self.modulation_amplitude)
        self.label = QtWidgets.QLabel(self.groupBox)
        self.label.setObjectName("label")
        self.horizontalLayout_2.addWidget(self.label)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.verticalLayout_9.addWidget(self.groupBox)
        self.groupBox_3 = QtWidgets.QGroupBox(self.scrollAreaWidgetContents_3)
        self.groupBox_3.setObjectName("groupBox_3")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.groupBox_3)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.ramp_speed = QtWidgets.QComboBox(self.groupBox_3)
        self.ramp_speed.setObjectName("ramp_speed")
        self.ramp_speed.addItem("")
        self.ramp_speed.addItem("")
        self.ramp_speed.addItem("")
        self.ramp_speed.addItem("")
        self.ramp_speed.addItem("")
        self.ramp_speed.addItem("")
        self.ramp_speed.addItem("")
        self.ramp_speed.addItem("")
        self.ramp_speed.addItem("")
        self.ramp_speed.addItem("")
        self.ramp_speed.addItem("")
        self.ramp_speed.addItem("")
        self.ramp_speed.addItem("")
        self.ramp_speed.addItem("")
        self.ramp_speed.addItem("")
        self.ramp_speed.addItem("")
        self.horizontalLayout_5.addWidget(self.ramp_speed)
        self.verticalLayout_4.addLayout(self.horizontalLayout_5)
        self.verticalLayout_9.addWidget(self.groupBox_3)
        self.groupBox_2 = QtWidgets.QGroupBox(self.scrollAreaWidgetContents_3)
        self.groupBox_2.setObjectName("groupBox_2")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox_2)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.demodulation_phase = QtWidgets.QDoubleSpinBox(self.groupBox_2)
        self.demodulation_phase.setMaximum(360.0)
        self.demodulation_phase.setSingleStep(10.0)
        self.demodulation_phase.setObjectName("demodulation_phase")
        self.horizontalLayout_4.addWidget(self.demodulation_phase)
        self.label_3 = QtWidgets.QLabel(self.groupBox_2)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_4.addWidget(self.label_3)
        self.verticalLayout_3.addLayout(self.horizontalLayout_4)
        self.verticalLayout_9.addWidget(self.groupBox_2)
        self.groupBox_7 = QtWidgets.QGroupBox(self.scrollAreaWidgetContents_3)
        self.groupBox_7.setObjectName("groupBox_7")
        self.verticalLayout_13 = QtWidgets.QVBoxLayout(self.groupBox_7)
        self.verticalLayout_13.setObjectName("verticalLayout_13")
        self.demodulation_frequency = QtWidgets.QComboBox(self.groupBox_7)
        self.demodulation_frequency.setObjectName("demodulation_frequency")
        self.demodulation_frequency.addItem("")
        self.demodulation_frequency.addItem("")
        self.demodulation_frequency.addItem("")
        self.demodulation_frequency.addItem("")
        self.demodulation_frequency.addItem("")
        self.verticalLayout_13.addWidget(self.demodulation_frequency)
        self.verticalLayout_9.addWidget(self.groupBox_7)
        self.groupBox_6 = QtWidgets.QGroupBox(self.scrollAreaWidgetContents_3)
        self.groupBox_6.setObjectName("groupBox_6")
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout(self.groupBox_6)
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.signal_offset = QtWidgets.QSpinBox(self.groupBox_6)
        self.signal_offset.setSingleStep(100)
        self.signal_offset.setObjectName("signal_offset")
        self.horizontalLayout_7.addWidget(self.signal_offset)
        self.verticalLayout_9.addWidget(self.groupBox_6)
        self.scrollArea_2.setWidget(self.scrollAreaWidgetContents_3)
        self.verticalLayout.addWidget(self.scrollArea_2)
        self.toolBox.addItem(self.page, "")
        self.page_3 = QtWidgets.QWidget()
        self.page_3.setGeometry(QtCore.QRect(0, 0, 104, 31))
        self.page_3.setObjectName("page_3")
        self.verticalLayout_14 = QtWidgets.QVBoxLayout(self.page_3)
        self.verticalLayout_14.setObjectName("verticalLayout_14")
        self.label_7 = QtWidgets.QLabel(self.page_3)
        self.label_7.setObjectName("label_7")
        self.verticalLayout_14.addWidget(self.label_7)
        self.toolBox.addItem(self.page_3, "")
        self.lockingPanel = LockingPanel()
        self.lockingPanel.setGeometry(QtCore.QRect(0, 0, 250, 498))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lockingPanel.sizePolicy().hasHeightForWidth())
        self.lockingPanel.setSizePolicy(sizePolicy)
        self.lockingPanel.setObjectName("lockingPanel")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.lockingPanel)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.scrollArea = QtWidgets.QScrollArea(self.lockingPanel)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 232, 730))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.scrollAreaWidgetContents)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.groupBox_5 = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_5.sizePolicy().hasHeightForWidth())
        self.groupBox_5.setSizePolicy(sizePolicy)
        self.groupBox_5.setObjectName("groupBox_5")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.groupBox_5)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.kp = QtWidgets.QSpinBox(self.groupBox_5)
        self.kp.setMinimum(0)
        self.kp.setMaximum(8191)
        self.kp.setObjectName("kp")
        self.verticalLayout_6.addWidget(self.kp)
        self.ki = QtWidgets.QSpinBox(self.groupBox_5)
        self.ki.setMinimum(0)
        self.ki.setMaximum(8191)
        self.ki.setObjectName("ki")
        self.verticalLayout_6.addWidget(self.ki)
        self.kd = QtWidgets.QSpinBox(self.groupBox_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.kd.sizePolicy().hasHeightForWidth())
        self.kd.setSizePolicy(sizePolicy)
        self.kd.setMinimum(0)
        self.kd.setMaximum(8191)
        self.kd.setObjectName("kd")
        self.verticalLayout_6.addWidget(self.kd)
        self.verticalLayout_8.addWidget(self.groupBox_5)
        self.lock_control_container = QtWidgets.QTabWidget(self.scrollAreaWidgetContents)
        self.lock_control_container.setObjectName("lock_control_container")
        self.auto_mode = QtWidgets.QWidget()
        self.auto_mode.setObjectName("auto_mode")
        self.verticalLayout_10 = QtWidgets.QVBoxLayout(self.auto_mode)
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        self.label_2 = QtWidgets.QLabel(self.auto_mode)
        self.label_2.setScaledContents(False)
        self.label_2.setAlignment(QtCore.Qt.AlignCenter)
        self.label_2.setWordWrap(True)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_10.addWidget(self.label_2)
        self.watchLockCheckbox = QtWidgets.QCheckBox(self.auto_mode)
        self.watchLockCheckbox.setChecked(True)
        self.watchLockCheckbox.setObjectName("watchLockCheckbox")
        self.verticalLayout_10.addWidget(self.watchLockCheckbox)
        self.checkBox_2 = QtWidgets.QCheckBox(self.auto_mode)
        self.checkBox_2.setChecked(True)
        self.checkBox_2.setObjectName("checkBox_2")
        self.verticalLayout_10.addWidget(self.checkBox_2)
        self.checkBox_4 = QtWidgets.QCheckBox(self.auto_mode)
        self.checkBox_4.setObjectName("checkBox_4")
        self.verticalLayout_10.addWidget(self.checkBox_4)
        self.checkBox_3 = QtWidgets.QCheckBox(self.auto_mode)
        self.checkBox_3.setObjectName("checkBox_3")
        self.verticalLayout_10.addWidget(self.checkBox_3)
        self.lock_control_container.addTab(self.auto_mode, "")
        self.manual_mode = QtWidgets.QWidget()
        self.manual_mode.setObjectName("manual_mode")
        self.verticalLayout_12 = QtWidgets.QVBoxLayout(self.manual_mode)
        self.verticalLayout_12.setObjectName("verticalLayout_12")
        self.label_6 = QtWidgets.QLabel(self.manual_mode)
        self.label_6.setScaledContents(False)
        self.label_6.setAlignment(QtCore.Qt.AlignCenter)
        self.label_6.setWordWrap(True)
        self.label_6.setObjectName("label_6")
        self.verticalLayout_12.addWidget(self.label_6)
        self.button_slope_rising = QtWidgets.QRadioButton(self.manual_mode)
        self.button_slope_rising.setObjectName("button_slope_rising")
        self.verticalLayout_12.addWidget(self.button_slope_rising)
        self.button_slope_falling = QtWidgets.QRadioButton(self.manual_mode)
        self.button_slope_falling.setObjectName("button_slope_falling")
        self.verticalLayout_12.addWidget(self.button_slope_falling)
        self.manualLockButton = QtWidgets.QPushButton(self.manual_mode)
        self.manualLockButton.setObjectName("manualLockButton")
        self.verticalLayout_12.addWidget(self.manualLockButton)
        self.lock_control_container.addTab(self.manual_mode, "")
        self.verticalLayout_8.addWidget(self.lock_control_container)
        self.lock_status_container = LockStatusPanel(self.scrollAreaWidgetContents)
        self.lock_status_container.setObjectName("lock_status_container")
        self.verticalLayout_18 = QtWidgets.QVBoxLayout(self.lock_status_container)
        self.verticalLayout_18.setObjectName("verticalLayout_18")
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_18.addItem(spacerItem1)
        self.lock_status = QtWidgets.QLabel(self.lock_status_container)
        self.lock_status.setWordWrap(True)
        self.lock_status.setObjectName("lock_status")
        self.verticalLayout_18.addWidget(self.lock_status)
        self.stop_lock = QtWidgets.QPushButton(self.lock_status_container)
        self.stop_lock.setObjectName("stop_lock")
        self.verticalLayout_18.addWidget(self.stop_lock)
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_18.addItem(spacerItem2)
        self.groupBox_8 = QtWidgets.QGroupBox(self.lock_status_container)
        self.groupBox_8.setObjectName("groupBox_8")
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout(self.groupBox_8)
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.control_signal_history_length = QtWidgets.QSpinBox(self.groupBox_8)
        self.control_signal_history_length.setMinimum(10)
        self.control_signal_history_length.setMaximum(100000)
        self.control_signal_history_length.setSingleStep(60)
        self.control_signal_history_length.setProperty("value", 120)
        self.control_signal_history_length.setObjectName("control_signal_history_length")
        self.horizontalLayout_9.addWidget(self.control_signal_history_length)
        self.label_4 = QtWidgets.QLabel(self.groupBox_8)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_9.addWidget(self.label_4)
        self.verticalLayout_18.addWidget(self.groupBox_8)
        spacerItem3 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_18.addItem(spacerItem3)
        self.verticalLayout_8.addWidget(self.lock_status_container)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.verticalLayout_7.addWidget(self.scrollArea)
        self.toolBox.addItem(self.lockingPanel, "")
        self.verticalLayout_11.addWidget(self.toolBox)
        self.horizontalLayout.addWidget(self.rightPanel)
        MainWindow.setCentralWidget(self.centralwidget)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.retranslateUi(MainWindow)
        self.toolBox.setCurrentIndex(2)
        self.lock_control_container.setCurrentIndex(0)
        self.closeButton.clicked.connect(self.rightPanel.close_app)
        self.shutdownButton.clicked.connect(self.rightPanel.shutdown_server)
        self.openDeviceManagerButton.clicked.connect(self.rightPanel.open_device_manager)
        self.decrease_scan_amplitude.clicked.connect(self.centralPanel.decrease_scan_amplitude)
        self.increase_scan_amplitude.clicked.connect(self.centralPanel.increase_scan_amplitude)
        self.reset_scan_amplitude.clicked.connect(self.centralPanel.reset_scan_amplitude)
        self.go_left.clicked.connect(self.centralPanel.go_left)
        self.go_right.clicked.connect(self.centralPanel.go_right)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow"))
        self.scan_amplitude.setText(_translate("MainWindow", "100 %"))
        self.decrease_scan_amplitude.setText(_translate("MainWindow", "+"))
        self.increase_scan_amplitude.setText(_translate("MainWindow", "-"))
        self.reset_scan_amplitude.setText(_translate("MainWindow", "reset"))
        self.go_left.setText(_translate("MainWindow", "<"))
        self.go_right.setText(_translate("MainWindow", ">"))
        self.openDeviceManagerButton.setText(_translate("MainWindow", "Device manager"))
        self.closeButton.setText(_translate("MainWindow", "Close client"))
        self.shutdownButton.setText(_translate("MainWindow", "Shutdown server"))
        self.groupBox_4.setTitle(_translate("MainWindow", "Modulation frequency"))
        self.label_5.setText(_translate("MainWindow", "MHz"))
        self.groupBox.setTitle(_translate("MainWindow", "Modulation amplitude"))
        self.label.setText(_translate("MainWindow", "Vpp"))
        self.groupBox_3.setTitle(_translate("MainWindow", "Ramp speed"))
        self.ramp_speed.setItemText(0, _translate("MainWindow", "3.8 kHz"))
        self.ramp_speed.setItemText(1, _translate("MainWindow", "1.9 kHz"))
        self.ramp_speed.setItemText(2, _translate("MainWindow", "954 Hz"))
        self.ramp_speed.setItemText(3, _translate("MainWindow", "477 Hz"))
        self.ramp_speed.setItemText(4, _translate("MainWindow", "238 Hz"))
        self.ramp_speed.setItemText(5, _translate("MainWindow", "119 Hz"))
        self.ramp_speed.setItemText(6, _translate("MainWindow", "59 Hz"))
        self.ramp_speed.setItemText(7, _translate("MainWindow", "30 Hz"))
        self.ramp_speed.setItemText(8, _translate("MainWindow", "15 Hz"))
        self.ramp_speed.setItemText(9, _translate("MainWindow", "7.5 Hz"))
        self.ramp_speed.setItemText(10, _translate("MainWindow", "3.7 Hz"))
        self.ramp_speed.setItemText(11, _translate("MainWindow", "1.9 Hz"))
        self.ramp_speed.setItemText(12, _translate("MainWindow", "0.93 Hz"))
        self.ramp_speed.setItemText(13, _translate("MainWindow", "0.47 Hz"))
        self.ramp_speed.setItemText(14, _translate("MainWindow", "0.23 Hz"))
        self.ramp_speed.setItemText(15, _translate("MainWindow", "0.12 Hz"))
        self.groupBox_2.setTitle(_translate("MainWindow", "Demodulation phase"))
        self.label_3.setText(_translate("MainWindow", "deg"))
        self.groupBox_7.setTitle(_translate("MainWindow", "Demodulation frequency"))
        self.demodulation_frequency.setItemText(0, _translate("MainWindow", "1f"))
        self.demodulation_frequency.setItemText(1, _translate("MainWindow", "2f"))
        self.demodulation_frequency.setItemText(2, _translate("MainWindow", "3f"))
        self.demodulation_frequency.setItemText(3, _translate("MainWindow", "4f"))
        self.demodulation_frequency.setItemText(4, _translate("MainWindow", "5f"))
        self.groupBox_6.setTitle(_translate("MainWindow", "Signal offset"))
        self.toolBox.setItemText(self.toolBox.indexOf(self.page), _translate("MainWindow", "Spectroscopy"))
        self.label_7.setText(_translate("MainWindow", "still missing..."))
        self.toolBox.setItemText(self.toolBox.indexOf(self.page_3), _translate("MainWindow", "Signal filtering"))
        self.groupBox_5.setTitle(_translate("MainWindow", "P,I and D"))
        self.label_2.setText(_translate("MainWindow", "<html><head/><body><p align=\"center\"><span style=\" font-size:14pt; font-weight:600;\">For automatic lock, click and drag over a line.</span></p></body></html>"))
        self.watchLockCheckbox.setText(_translate("MainWindow", "Watch lock"))
        self.checkBox_2.setText(_translate("MainWindow", "Determine signal offset"))
        self.checkBox_4.setText(_translate("MainWindow", "Emit TTL when locked"))
        self.checkBox_3.setText(_translate("MainWindow", "Relock on TTL"))
        self.lock_control_container.setTabText(self.lock_control_container.indexOf(self.auto_mode), _translate("MainWindow", "Auto"))
        self.label_6.setText(_translate("MainWindow", "<html><head/><body><p align=\"center\"><span style=\" font-size:14pt; font-weight:600;\">Center the line and zoom in before pressing the button.</span></p></body></html>"))
        self.button_slope_rising.setText(_translate("MainWindow", "Rising slope"))
        self.button_slope_falling.setText(_translate("MainWindow", "Falling slope"))
        self.manualLockButton.setText(_translate("MainWindow", "Lock!"))
        self.lock_control_container.setTabText(self.lock_control_container.indexOf(self.manual_mode), _translate("MainWindow", "Manual"))
        self.lock_status.setText(_translate("MainWindow", "<html><head/><body><p align=\"center\"><span style=\" font-size:16pt; font-weight:600;\">Lock status</span></p></body></html>"))
        self.stop_lock.setText(_translate("MainWindow", "Stop lock"))
        self.groupBox_8.setTitle(_translate("MainWindow", "Control signal history length"))
        self.label_4.setText(_translate("MainWindow", "s"))
        self.toolBox.setItemText(self.toolBox.indexOf(self.lockingPanel), _translate("MainWindow", "Locking"))


from central_panel import CentralPanel
from lock_status_panel import LockStatusPanel
from locking_panel import LockingPanel
from plot_widget import PlotWidget
from right_panel import RightPanel
from spectroscopy_panel import SpectroscopyPanel


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    MainWindow = QtWidgets.QMainWindow()
    ui = Ui_MainWindow()
    ui.setupUi(MainWindow)
    MainWindow.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'device_manager.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DeviceManager(object):
    def setupUi(self, DeviceManager):
        DeviceManager.setObjectName("DeviceManager")
        DeviceManager.resize(510, 331)
        self.centralwidget = DeviceManagerCenter(DeviceManager)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.centralwidget.sizePolicy().hasHeightForWidth())
        self.centralwidget.setSizePolicy(sizePolicy)
        self.centralwidget.setMinimumSize(QtCore.QSize(510, 280))
        self.centralwidget.setObjectName("centralwidget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.centralwidget)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.deviceList = QtWidgets.QListWidget(self.centralwidget)
        self.deviceList.setObjectName("deviceList")
        self.horizontalLayout_2.addWidget(self.deviceList)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.connectButton = QtWidgets.QPushButton(self.centralwidget)
        self.connectButton.setEnabled(False)
        self.connectButton.setObjectName("connectButton")
        self.verticalLayout.addWidget(self.connectButton)
        self.addButton = QtWidgets.QPushButton(self.centralwidget)
        self.addButton.setObjectName("addButton")
        self.verticalLayout.addWidget(self.addButton)
        self.removeButton = QtWidgets.QPushButton(self.centralwidget)
        self.removeButton.setEnabled(False)
        self.removeButton.setObjectName("removeButton")
        self.verticalLayout.addWidget(self.removeButton)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.horizontalLayout_2.addLayout(self.verticalLayout)
        self.horizontalLayout.addLayout(self.horizontalLayout_2)
        self.menubar = QtWidgets.QMenuBar(DeviceManager)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 510, 27))
        self.menubar.setObjectName("menubar")
        DeviceManager.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(DeviceManager)
        self.statusbar.setObjectName("statusbar")
        DeviceManager.setStatusBar(self.statusbar)

        self.retranslateUi(DeviceManager)
        self.connectButton.clicked.connect(self.centralwidget.connect)
        self.addButton.clicked.connect(self.centralwidget.new_device)
        self.removeButton.clicked.connect(self.centralwidget.remove_device)
        self.deviceList.currentRowChanged['int'].connect(self.centralwidget.selected_device_changed)
        QtCore.QMetaObject.connectSlotsByName(DeviceManager)

    def retranslateUi(self, DeviceManager):
        _translate = QtCore.QCoreApplication.translate
        DeviceManager.setWindowTitle(_translate("DeviceManager", "MainWindow"))
        self.connectButton.setText(_translate("DeviceManager", "Connect"))
        self.addButton.setText(_translate("DeviceManager", "New device"))
        self.removeButton.setText(_translate("DeviceManager", "Remove device"))


from device_manager_center import DeviceManagerCenter


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    DeviceManager = QtWidgets.QMainWindow()
    ui = Ui_DeviceManager()
    ui.setupUi(DeviceManager)
    DeviceManager.show()
    sys.exit(app.exec_())
